@@ -1,21 +1,37 @@
 /*
+
+$Id: compress.c,v 1.70 2005/05/12 15:41:04 karman Exp $
+
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+** karman: how much of this file is still original??
+
+
+
 ** 2001-02-12 rasc   errormsg "print" changed...
 **
 */
@@ -36,6 +52,7 @@
 
 #ifdef HAVE_ZLIB
 #include <zlib.h>
+#define Z_BUFSIZE 16384
 #endif
 
 /* Surfing the web I found this:
@@ -151,12 +168,22 @@ int     uncompress1(FILE * fp, int (*f_getc) (FILE *))
 {
     int     _c;
     int     num = 0;
+    
+    /* printf("uncompress: _c = %d num = %d\n", _c, num); */
 
     do
     {
         _c = (int) f_getc(fp);
+        
+        if (_c < 0) {
+             progerr("_c is < 0 in uncompress1()\n");
+        }
+        
         num <<= 7;
         num |= _c & 127;
+        
+        /* printf("uncompress: _c = %d num = %d\n", _c, num); */
+        
         if (!num)
             break;
     }
@@ -187,7 +214,7 @@ int     uncompress2(unsigned char **buffer)
     return num;
 }
 
-/* Rourtines to make long integers portable */
+/* Routines to make long integers portable */
 unsigned long PACKLONG(unsigned long num)
 {
     unsigned long tmp = 0L;
@@ -255,7 +282,74 @@ unsigned long UNPACKLONG2(unsigned char *s)
     return tmp;
 }
 
+/* 2003/10/28 jmruiz - Routines to make file offsets portable */
+/* sw_off_t is a type defined in config.h to be 32 or 64 bit */
+sw_off_t PACKFILEOFFSET(sw_off_t num)
+{
+    sw_off_t tmp = (sw_off_t)0;
+    unsigned char *s;
+    int sz_off_t = sizeof(sw_off_t);
 
+    if (num && LITTLE_ENDIAN)
+    {
+        s = (unsigned char *) &tmp;
+        while(sz_off_t)
+            *s++ = (unsigned char) ((num >> (sw_off_t)((--sz_off_t)<<3)) & (sw_off_t)0xFF);
+
+        return tmp;
+    }
+    return num;
+}
+
+/* Same routine - Packs file offset into a buffer */
+void    PACKFILEOFFSET2(sw_off_t num, unsigned char *s)
+{
+    int sz_off_t = sizeof(sw_off_t);
+
+    if(LITTLE_ENDIAN)
+    {
+        while(sz_off_t)
+            *s++ = (unsigned char) ((num >> (sw_off_t)((--sz_off_t)<<3)) & (sw_off_t)0xFF);
+    }
+    else
+    {
+        memcpy(s,(unsigned char *)&num,sz_off_t);
+    }
+}
+
+/* Routine to unpack a file offset */
+sw_off_t UNPACKFILEOFFSET(sw_off_t num)
+{
+    int sz_off_t = sizeof(sw_off_t);
+    sw_off_t tmp = (sw_off_t)0;
+    unsigned char *s = (unsigned char *) &num;
+
+    if(LITTLE_ENDIAN)
+    {
+        while(sz_off_t)
+            tmp += (sw_off_t)((sw_off_t)(*s++) << (sw_off_t)((--sz_off_t)<<3));
+        return tmp;
+    }
+    return num;
+}
+
+/* Same routine - UnPacks file offset from buffer */
+sw_off_t UNPACKFILEOFFSET2(unsigned char *s)
+{
+    int sz_off_t = sizeof(sw_off_t);
+    sw_off_t tmp = (sw_off_t)0;
+
+    if(LITTLE_ENDIAN)
+    {
+        while(sz_off_t)
+            tmp += (sw_off_t)((sw_off_t)(*s++) << (sw_off_t)((--sz_off_t)<<3));
+    }
+    else
+    {
+        memcpy((unsigned char *)&tmp,s,sz_off_t);
+    }
+    return tmp;
+}
 
 /***********************************************************************************
 *   09/00 Jose Ruiz 
@@ -319,7 +413,7 @@ I wonder if storing the initial postion would improve that much.
 
 
 
-void compress_location_values(unsigned char **buf,unsigned char **flagp,int filenum,int frequency, int *posdata)
+void compress_location_values(unsigned char **buf,unsigned char **flagp,int filenum,int frequency, unsigned int *posdata)
 {
     unsigned char *p = *buf;
     unsigned char *flag;
@@ -387,7 +481,7 @@ void compress_location_values(unsigned char **buf,unsigned char **flagp,int file
     *buf = p;
 }
 
-void compress_location_positions(unsigned char **buf,unsigned char *flag,int frequency, int *posdata)
+void compress_location_positions(unsigned char **buf,unsigned char *flag,int frequency, unsigned int *posdata)
 {
     unsigned char *p = *buf;
     int i, j;
@@ -526,7 +620,7 @@ unsigned long four_bit_called = 0;
 unsigned long not_four_called;
 
 
-void uncompress_location_positions(unsigned char **buf, unsigned char flag, int frequency, int *posdata)
+void uncompress_location_positions(unsigned char **buf, unsigned char flag, int frequency, unsigned int *posdata)
 {
     int i, j, tmp;
     unsigned char *p = *buf;
@@ -562,7 +656,7 @@ void uncompress_location_positions(unsigned char **buf, unsigned char flag, int
         }
 
         /* First position is always "as is" */
-        posdata[0] = uncompress2(&p);
+        posdata[0] = (unsigned int)uncompress2(&p);
 
         /* Check if positions where stored as two values per byte or the old "compress" style */
         if(flag & POS_4_BIT)
@@ -570,9 +664,9 @@ void uncompress_location_positions(unsigned char **buf, unsigned char flag, int
             for (i = 1, j = 0; i < frequency; i++, j++)
             {
                 if(j%2)
-                    posdata[i] = p[j/2] & 0x0F;
+                    posdata[i] = (unsigned int)((unsigned int)p[j/2] & (unsigned int)0x0F);
                 else
-                    posdata[i] = p[j/2] >> 4;
+                    posdata[i] = (unsigned int)((unsigned int)p[j/2] >> (unsigned int)4);
             }
             p += ((j + 1)/2);
         }
@@ -581,7 +675,7 @@ void uncompress_location_positions(unsigned char **buf, unsigned char flag, int
             for (i = 1; i < frequency; i++)
             {
                 tmp = uncompress2(&p);
-                posdata[i] = tmp;
+                posdata[i] = (unsigned int)tmp;
             }
         }
         /* Position were compressed incrementally. So restore them */
@@ -626,10 +720,26 @@ void    CompressCurrentLocEntry(SWISH * sw, ENTRY * e)
 
 /* 2002/11 jmruiz
 ** Simple routine to compress worddata using zlib where available
+** 
+** 2004/06 jmruiz 
+** economic flag is for use less RAM
+** Trying to compress worddata needs some extra RAM in order to call
+** zlib's compress2 routine because this routine needs a buffer
+** for storing the compressed data.
+** So, if someone is trying to index in economic mode (-e switch)
+** he can experiment the annoying "Out of RAM" message if his computer
+** does not have enough RAM for allocating that buffer.
+** In order to fix it, I have tried the low level deflate routines of zlib's
+** (deflateInit, deflate and deflateEnd) with two local buffers:
+** local_buffer_in and local_buffer_out.
+** The original data is being copied in chunks to local_buffer_in
+** and compressed to local_buffer_out after each call to zlib's deflate
+** routine. The compressed chunks are ithen copied to the original worddata
+** area taking care of not to overrun the buffer.
 **
 ** On exit returns the new size of the compressed buffer
 */
-int compress_worddata(unsigned char *wdata,int wdata_size)
+int compress_worddata(unsigned char *wdata,int wdata_size, int economic)
 {
 #ifndef HAVE_ZLIB
     return wdata_size;
@@ -637,40 +747,160 @@ int compress_worddata(unsigned char *wdata,int wdata_size)
     unsigned char  *WDataBuf;     /* For compressing and uncompressing */
     uLongf          dest_size;
     int             zlib_status = 0;
-    unsigned char   local_buffer[8192];/* Just to avoid emalloc/efree overhead*/
+    int             off_wdata, len_chunk, off_out;
+    unsigned char   local_buffer_out[Z_BUFSIZE];/* Just to avoid emalloc/efree overhead and for deflate method */
+    unsigned char   local_buffer_in[Z_BUFSIZE];/* Just to avoid emalloc/efree overhead*/
 
 
     /* Don't bother compressing smaller items */
     if ( wdata_size < MIN_WORDDATA_COMPRESS_SIZE )
-    {
         return wdata_size;
-    }
 
-    /* Buffer should be +1% + a few bytes. */
-    dest_size = (uLongf)(wdata_size + ( wdata_size / 100 ) + 1000);  // way more than should be needed
+    if(economic)
+    {                    /* -e switch is set. Use deflate* routines */
+        z_stream z;      /* zlib compression stream */
 
-    /* Get an output buffer */
-    if( dest_size > sizeof(local_buffer) )
-        WDataBuf = (unsigned char *) emalloc((int)dest_size );
-    else
-        WDataBuf = local_buffer;
+        z.zalloc = (alloc_func)0;   /* init zlib compression stream */
+        z.zfree = (free_func)0;
+        z.opaque = (voidpf)0;
 
-    zlib_status = compress2((Bytef *)WDataBuf, &dest_size, wdata, wdata_size, 9);
-    if ( zlib_status != Z_OK )
-        progerr("WordData Compression Error.  zlib compress2 returned: %d  Worddata size: %d compress buf size: %d", zlib_status, wdata_size, (int)dest_size);
+        if(Z_OK != deflateInit(&z, 9))
+            return wdata_size;
 
-    /* Make sure it's compressed enough -- should check that destsize is not > MAXINT */
-    if ( (int)dest_size < wdata_size )
-    {
-        memcpy(wdata,WDataBuf,(int)dest_size);
+        z.avail_in  = 0;
+        z.next_out = (Bytef*)local_buffer_out;
+        z.avail_out  = Z_BUFSIZE;
+
+        dest_size = 0;
+        off_wdata = 0;
+        off_out = 0;
+        for(;;)
+        {
+            if (off_wdata == wdata_size)
+                break;     /* No more data */ 
+            else
+            {
+                if (z.avail_in==0) 
+                {            /* Fill local_buffer_in with more data */
+                    len_chunk = Min(Z_BUFSIZE,(wdata_size - off_wdata));
+                    if(!len_chunk)  /* No more data to compress: exit */
+                        break;     
+                    memcpy(local_buffer_in,wdata + off_wdata, len_chunk);
+                    off_wdata += len_chunk;
+                    z.next_in = local_buffer_in;
+                    z.avail_in = len_chunk;
+                }
+            }
+                       /* Compress local_buffer_in */
+                       /* Z_NO_FLUSH flag achieves better results */
+            zlib_status = deflate(&z, Z_NO_FLUSH);
+                       /* get the size of compressed data */
+            len_chunk = Z_BUFSIZE - z.avail_out;
+            if(len_chunk)
+            {
+                       /* Check for buffer overrun */
+                if((off_out + len_chunk) >= off_wdata)
+                {
+                    /* We are in buffer overrun condition but if we are in 
+                    ** the first chunk we can recover the original data
+                    ** from local_buffer_in
+                    */
+                    if(off_wdata <= Z_BUFSIZE)
+                    {
+                        deflateEnd(&z);
+                        memcpy(wdata,local_buffer_in,wdata_size);   /* Do nothing - Retains data uncompressed */
+                        return wdata_size;   
+                    }
+                    else
+                        progerr("WordData Compression Error. Unable to compress worddata in economic mode. Remove switch -e from your command line or add \"CompressPositions Yes\" to your config file");
+                }
+                       /* Copy the compressed data onto the original buffer */
+                       /* off_out contains the current length of the total 
+                       ** compressed data
+                       */
+                memcpy(wdata + off_out, local_buffer_out, len_chunk);
+                off_out += len_chunk;
+            }
+                       /* reset local_buffer_out to next step */
+            z.next_out = (Bytef*)local_buffer_out;
+            z.avail_out  = Z_BUFSIZE;
+            if(zlib_status != Z_OK)
+                break;
+        }
+
+                       /* We have used Z_NO_FLUSH to achieve better
+                       ** results. So, we have to issue a deflate with
+                       ** Z_FINISH flag to flush the pending data
+                       ** in local_buffer_out
+                       */
+        for(;;)
+        {
+            zlib_status = deflate(&z, Z_FINISH);
+                       /* get the size of compressed data */
+            len_chunk = Z_BUFSIZE - z.avail_out;
+            if(len_chunk)
+            {
+                       /* Check for buffer overrun */
+                if((off_out + len_chunk) >= off_wdata)
+                {
+                    /* We are in buffer overrun condition but if we are in 
+                    ** the first chunk we can recover the original data
+                    ** from local_buffer_in
+                    */
+                    if(off_wdata <= Z_BUFSIZE)
+                    {
+                        deflateEnd(&z);
+                        memcpy(wdata,local_buffer_in,wdata_size);   /* Do nothing - Retains data uncompressed */
+                        return wdata_size;   
+                    }
+                    else
+                        progerr("WordData Compression Error. Unable to compress worddata in economic mode. Remove switch -e from your command line or add \"CompressPositions Yes\" to your config file");
+                }
+                       /* Copy the compressed data onto the original buffer */
+                       /* off_out contains the current length of the total 
+                       ** compressed data
+                       */
+                memcpy(wdata + off_out, local_buffer_out, len_chunk);
+                off_out += len_chunk;
+            }
+                       /* reset local_buffer_out to next step */
+            z.next_out = (Bytef*)local_buffer_out;
+            z.avail_out  = Z_BUFSIZE;
+            if(zlib_status != Z_OK)
+                break;
+        }
+        deflateEnd(&z); 
+        dest_size = off_out;
     }
     else
     {
-        dest_size = wdata_size;
+        /* Buffer should be +1% + a few bytes. */
+        dest_size = (uLongf)(wdata_size + ( wdata_size / 100 ) + 1000);  // way more than should be needed
+
+        /* Get an output buffer */
+        if( dest_size > Z_BUFSIZE )
+            WDataBuf = (unsigned char *) emalloc((int)dest_size );
+        else
+            WDataBuf = local_buffer_out;
+
+        zlib_status = compress2((Bytef *)WDataBuf, &dest_size, wdata, wdata_size, 9);
+        if ( zlib_status != Z_OK )
+            progerr("WordData Compression Error.  zlib compress2 returned: %d  Worddata size: %d compress buf size: %d", zlib_status, wdata_size, (int)dest_size);
+
+        /* Make sure it's compressed enough -- should check that destsize is not > MAXINT */
+        if ( (int)dest_size < wdata_size )
+        {
+            memcpy(wdata,WDataBuf,(int)dest_size);
+        }
+        else
+        {
+            dest_size = wdata_size;
+        }
+
+        if ( WDataBuf != local_buffer_out)
+            efree(WDataBuf);
     }
 
-    if ( WDataBuf != local_buffer)
-        efree(WDataBuf);
     return (int)dest_size;
 #endif
 }
@@ -693,8 +923,8 @@ void uncompress_worddata(unsigned char **buf, int *buf_size, int saved_bytes)
     if ( zlib_status != Z_OK )
     {
         // $$$ make sure this works ok if returning null $$$
-        progwarn("Failed to uncompress Property. zlib uncompress returned: %d.  uncompressed size: %d buf_len: %d\n",
-            zlib_status, new_buf_size, buf_size );
+        progwarn("Failed to uncompress Property. zlib uncompress returned: %d.  uncompressed size: %d buf_len: %d saved_bytes: %d\n",
+            zlib_status, new_buf_size, *buf_size, saved_bytes );
         return;
     }
     efree(*buf);
