@@ -2,19 +2,33 @@
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+$Id: config.h,v 1.70 2006/08/24 04:06:28 karman Exp $
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 18:19:34 CDT 2005
+** added GPL
+
+
 **--------------------------------------------------------------------------
 ** Config file edited by Roy Tennant 2/20/96
 ** Config file edited by Giulia Hill 2/27/97 to increase lenght of
@@ -57,6 +71,8 @@
 
 /* MIN_PROP_COMPRESS_SIZE sets the limit for which properties are compressed 
  * must be compiled with zlib.
+ *
+ * NEAR WORD feature might benefit from setting these next 2 higher
  */
 #define MIN_PROP_COMPRESS_SIZE 100
 /* Same for worddata */
@@ -161,9 +177,10 @@
 ** silent operation.  The default before swish 2.2 was 3
 */
 
-#define _AND_WORD "and"
-#define _OR_WORD "or"
-#define _NOT_WORD "not"
+#define _NEAR_WORD  "near"
+#define _AND_WORD   "and"
+#define _OR_WORD    "or"
+#define _NOT_WORD   "not"
 
 /* 
  ** these are the default boolean operator words used by swish search
@@ -354,7 +371,9 @@
 
 /* 2001/10 jmruiz -- Added BTREE schema to store words */
 
-//#define USE_BTREE
+//#define USE_BTREE  /* use --enable-incremental at configure time */
+
+/* If USE_BTREE then enable the ARRAY code for the pre-sorted indexes */
 
 #define sw_fopen fopen
 #define sw_fclose fclose
