@@ -1,20 +1,30 @@
 /*
-**
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**
-**
+$Id: db.h,v 1.36 2005/05/12 15:41:04 karman Exp $
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 18:19:34 CDT 2005
+** added GPL
+
+
 **
 ** 2001-01  jose   initial coding
 **
@@ -34,7 +44,7 @@ DB_OPEN_MODE;
 void initModule_DB (SWISH *);
 void freeModule_DB (SWISH *);
 
-void    write_header(SWISH *, INDEXDATAHEADER *, void *, char *, int, int, int, int, int);
+void    write_header(SWISH *ws, int merged_flag );
 void    update_header(SWISH *, void *, int, int );
 void    write_index(SWISH *, IndexFILE *);
 void    write_word(SWISH *, ENTRY *, IndexFILE *);
@@ -58,11 +68,7 @@ void    parse_integer_table_from_buffer(int table[], int table_size, char *buffe
 char    *getfilewords(SWISH *sw, int, IndexFILE *);
 void    setTotalWordsPerFile(IndexFILE *,int ,int );
 
-#ifdef USE_BTREE
-void    getTotalWordsPerFile(SWISH *, IndexFILE *, int ,int *);
-#else
-void    getTotalWordsPerFile(IndexFILE *, int ,int *);
-#endif
+int     getTotalWordsInFile(IndexFILE *indexf, int filenum);
 
 
 /* Common DB api */
@@ -98,22 +104,36 @@ long    DB_ReadWordData(SWISH *sw, sw_off_t wordID, unsigned char **worddata, in
 int     DB_EndReadWords(SWISH *sw, void *DB);
 
 
-#ifdef USE_BTREE
+#ifdef USE_PRESORT_ARRAY
 int     DB_InitWriteSortedIndex(SWISH *sw, void *DB, int n_props );
+int     DB_WriteSortedIndex(SWISH *sw, int propID, int *data, int sz_data,void *DB);
 #else
 int     DB_InitWriteSortedIndex(SWISH *sw, void *DB );
-#endif
 int     DB_WriteSortedIndex(SWISH *sw, int propID, unsigned char *data, int sz_data,void *DB);
+#endif
+
 int     DB_EndWriteSortedIndex(SWISH *sw, void *DB);
  
 int     DB_InitReadSortedIndex(SWISH *sw, void *DB);
 int     DB_ReadSortedIndex(SWISH *sw, int propID, unsigned char **data, int *sz_data,void *DB);
+
+/* this is defined in db_native.h now 
 int     DB_ReadSortedData(SWISH *sw, int *data,int index, int *value, void *DB);
+*/
+#ifdef USE_PRESORT_ARRAY
+#define  DB_ReadSortedData(data, index) (ARRAY_Get((ARRAY *)data,index))
+#else
+#define  DB_ReadSortedData(data, index) (data[index])
+#endif
+
+
+
+
 int     DB_EndReadSortedIndex(SWISH *sw, void *DB);
 
 
 int     DB_WriteFileNum(SWISH *sw, int filenum, unsigned char *filedata,int sz_filedata, void *DB);
-int     DB_ReadFileNum(SWISH *sw, int *filenum, unsigned char *filedata,int sz_filedata, void *DB);
+int     DB_ReadFileNum(SWISH *sw, unsigned char *filedata, void *DB);
 int     DB_CheckFileNum(SWISH *sw, int filenum, void *DB);
 int     DB_RemoveFileNum(SWISH *sw, int filenum, void *DB);
 
@@ -167,16 +187,17 @@ struct MOD_DB
     
     
     int    (*DB_WriteFileNum) (int filenum, unsigned char *filedata,int sz_filedata, void *DB);
-    int    (*DB_ReadFileNum) (int *filenum, unsigned char *filedata,int sz_filedata, void *DB);
+    int    (*DB_ReadFileNum) ( unsigned char *filedata, void *DB);
     int    (*DB_CheckFileNum) (int filenum, void *DB);
     int    (*DB_RemoveFileNum) (int filenum, void *DB);
 
-#ifdef USE_BTREE
+#ifdef USE_PRESORT_ARRAY
     int    (*DB_InitWriteSortedIndex) (void *DB, int n_props);
+    int    (*DB_WriteSortedIndex) (int propID, int *data, int sz_data,void *DB);
 #else
     int    (*DB_InitWriteSortedIndex) (void *DB);
-#endif
     int    (*DB_WriteSortedIndex) (int propID, unsigned char *data, int sz_data,void *DB);
+#endif
     int    (*DB_EndWriteSortedIndex) (void *DB);
      
     int    (*DB_InitReadSortedIndex) (void *DB);
