@@ -1,20 +1,31 @@
 /*
-**
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**
-**
+
+$Id: db_native.c,v 1.91 2006/08/25 02:49:47 karman Exp $
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
+
 **
 ** 2001-05-07 jmruiz init coding
 **
@@ -39,6 +50,13 @@
 #define WRITE_WORDS_RAMDISK 1
 #endif
 
+/* MAX_PATH used by Herman's NEAR feature but it seems to be a Windoze thing 
+ * so karman just made this value up so it will compile on *nix
+ */
+#if !defined(_WIN32)
+#define MAX_PATH    255
+#endif
+
 // #define DEBUG_PROP 1
 
 /*
@@ -157,7 +175,7 @@ static void DB_CheckHeader(struct Handle_DBNative *DB)
 
 
 
-    {   
+    {
 #ifdef USE_BTREE
         long btree, worddata, hashfile, array, presorted;
 #endif
@@ -253,17 +271,17 @@ static struct Handle_DBNative *newNativeDBHandle(SWISH *sw, char *dbname)
 /* Open files */
 
 
-static FILE   *openIndexFILEForRead(char *filename) 
+static FILE   *openIndexFILEForRead(char *filename)
 {
     return sw_fopen(filename, F_READ_BINARY);
 }
 
 static FILE   *openIndexFILEForReadAndWrite(char *filename)
-{  
+{
     return sw_fopen(filename, F_READWRITE_BINARY);
 }
 
-  
+
 static FILE   *openIndexFILEForWrite(char *filename)
 {
     return sw_fopen(filename, F_WRITE_BINARY);
@@ -272,7 +290,7 @@ static FILE   *openIndexFILEForWrite(char *filename)
 static void    CreateEmptyFile(char *filename)
 {
     FILE   *fp;
-    
+
     if (!(fp = openIndexFILEForWrite(filename)))
     {
         progerrno("Couldn't write the file \"%s\": ", filename);
@@ -304,7 +322,6 @@ void   *DB_Create_Native(SWISH *sw, char *dbname)
 #endif
     struct Handle_DBNative *DB;
 
-
     if ( is_directory( dbname ) )
         progerr( "Index file '%s' is a directory", dbname );
 
@@ -523,7 +540,7 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
     DB->cur_prop_file = s;
 
 #ifdef USE_BTREE
-    
+
     s = emalloc(strlen(dbname) + strlen(BTREE_EXTENSION) + 1);
 
     strcpy(s, dbname);
@@ -534,10 +551,10 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
         set_progerrno(INDEX_FILE_ERROR, DB->sw, "Couldn't open the btree file \"%s\": ", s);
         return (void *) DB;
     }
-        
 
 
-    DB->fp_btree = fp_tmp;      
+
+    DB->fp_btree = fp_tmp;
     DB->cur_btree_file = s;
 
     s = emalloc(strlen(dbname) + strlen(PRESORTED_EXTENSION) + 1);
@@ -550,12 +567,12 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
         set_progerrno(INDEX_FILE_ERROR, DB->sw, "Couldn't open the presorted index file \"%s\": ", s);
         return (void *) DB;
     }
-        
+
 
     DB->cur_presorted_file = s;
-    
 
-    
+
+
     s = emalloc(strlen(dbname) + strlen(WORDDATA_EXTENSION) + 1);
 
     strcpy(s, dbname);
@@ -566,7 +583,7 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
         set_progerrno(INDEX_FILE_ERROR, DB->sw, "Couldn't open the worddata file \"%s\": ", s);
         return (void *) DB;
     }
-        
+
 
     DB->fp_worddata = fp_tmp;
     DB->cur_worddata_file = s;
@@ -582,7 +599,7 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
         set_progerrno(INDEX_FILE_ERROR, DB->sw, "Couldn't open the hashfile file \"%s\": ", s);
         return (void *) DB;
     }
-        
+
 
     DB->fp_hashfile = fp_tmp;
     DB->cur_hashfile_file = s;
@@ -598,7 +615,7 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
         set_progerrno(INDEX_FILE_ERROR, DB->sw, "Couldn't open the array file \"%s\": ", s);
         return (void *) DB;
     }
-          
+
     DB->fp_array = fp_tmp;
     DB->cur_array_file = s;
 
@@ -626,7 +643,7 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
     DB->totwords_array = ARRAY_Open(DB->fp_array,DB->offsets[TOTALWORDSPERFILEPOS]);
     DB->props_array = ARRAY_Open(DB->fp_array,DB->offsets[FILEOFFSETPOS]);
 
-    /* Put the file pointer of props file at the end of the file 
+    /* Put the file pointer of props file at the end of the file
     ** This is very important because if we are in update mode
     ** we must avoid the properties to be overwritten
     */
@@ -644,8 +661,8 @@ void   *DB_Open_Native(SWISH *sw, char *dbname,int mode)
 
 static void DB_Close_File_Native(FILE ** fp, char **filename, int *tempflag)
 {
-#ifdef _WIN32
-	struct stat stbuf;
+#if defined(_WIN32) && !defined(__CYGWIN__)
+        struct stat stbuf;
 #endif
     if (!*fp)
         return;
@@ -662,9 +679,9 @@ static void DB_Close_File_Native(FILE ** fp, char **filename, int *tempflag)
 
         newname[strlen(newname) - strlen(USE_TEMPFILE_EXTENSION)] = '\0';
 
-#ifdef _WIN32
-	if(!stat(newname, &stbuf) && ((stbuf.st_mode & S_IFMT) == S_IFREG))
-	/* if(isfile(newname)) FIXME: file.c shouldn't rely on indexing structures */
+#if defined(_WIN32) && !defined(__CYGWIN__)
+        if(!stat(newname, &stbuf) && ((stbuf.st_mode & S_IFMT) == S_IFREG))
+        /* if(isfile(newname)) FIXME: file.c shouldn't rely on indexing structures */
             if (remove(newname))
                 progerrno("Failed to unlink '%s' before renaming. : ", newname);
 #endif
@@ -815,9 +832,9 @@ int     DB_InitWriteHeader_Native(void *db)
 
     if(DB->offsets[HEADERPOS])
     {
-        /* If DB->offsets[HEADERPOS] is not 0 we are in update mode 
+        /* If DB->offsets[HEADERPOS] is not 0 we are in update mode
         ** So, put the pointer file in the header start position to overwrite
-        ** the header 
+        ** the header
         */
         sw_fseek(DB->fp,DB->offsets[HEADERPOS],SEEK_SET);
     }
@@ -962,6 +979,9 @@ int     DB_EndWriteWords_Native(void *db)
     /* Now update word's data offset into the list of words */
     /* Simple check  words and worddata must match */
 
+    if (! DB->num_words)
+        progerr("No unique words indexed");
+
     if (DB->num_words != DB->wordhash_counter)
         progerrno("Internal DB_native error - DB->num_words != DB->wordhash_counter: ");
 
@@ -1577,7 +1597,7 @@ long    DB_ReadWordData_Native(sw_off_t wordID, unsigned char **worddata, int *d
     /* Get saved_bytes and adjust data_size */
     *saved_bytes = uncompress2(&buf);
     *data_size -= (buf - (*worddata));
-    /* Remove saved_bytes from buffer 
+    /* Remove saved_bytes from buffer
     ** We need to use memmove because data overlaps */
     memmove(*worddata,buf, *data_size);
 
@@ -1588,8 +1608,8 @@ long    DB_ReadWordData_Native(sw_off_t wordID, unsigned char **worddata, int *d
 
 
 /*--------------------------------------------
-** 2002/12 Jose Ruiz 
-**     FilePath,FileNum  pairs              
+** 2002/12 Jose Ruiz
+**     FilePath,FileNum  pairs
 **     Auxiliar hash index
 */
 
@@ -1611,19 +1631,19 @@ int     DB_WriteFileNum_Native(int filenum, unsigned char *filedata, int sz_file
 }
 
 /* Routine to get filenum from path */
-int     DB_ReadFileNum_Native(int *filenum, unsigned char *filedata, int sz_filedata, void *db)
+int     DB_ReadFileNum_Native(unsigned char *filedata, void *db)
 {
 #ifdef USE_BTREE
     unsigned long tmp;
     struct Handle_DBNative *DB = (struct Handle_DBNative *) db;
 
     /* Write it to the hash index */
-    FHASH_Search(DB->hashfile, filedata, sz_filedata, (unsigned char *)&tmp, sizeof(unsigned long));
+    FHASH_Search(DB->hashfile, filedata, strlen(filedata), (unsigned char *)&tmp, sizeof(unsigned long));
 
     /* UnPack tmp */
     tmp = PACKLONG(tmp);
 
-    *filenum = (int)tmp;
+    return (int)tmp;
 #endif
     return 0;
 }
@@ -1660,7 +1680,70 @@ int     DB_RemoveFileNum_Native(int filenum, void *db)
 /*--------------------------------------------*/
 /*--------------------------------------------*/
 
-#ifdef USE_BTREE
+#ifdef USE_PRESORT_ARRAY
+
+/*******************************************************************************
+*  Sorted data tables (presorted indexes)
+*
+*  DB->offsets[SORTEDINDEX]     Points to start of presorted index tables
+*  DB->n_presorted_array        Number of props in index
+*  DB->presorted_array          Points to array[ # props ] of pointers to ARRAY
+*                               ARRAY is defined in array.h
+*  DB->presorted_root_node      Points to array[ # props ] of unsigned longs
+*                               holds the seek pointer to the root node for
+*                               each of the sorted props.
+*  DB->next_sortedindex         Counter - index into structure below.
+*                               incremented after each write of an array.
+*
+*  Index layout:
+*            +-------------------------------+
+*            |  Num props                    |  <<-- DB->offsets[SORTEDINEDEX]
+*            +-------------------------------+
+*            |  PropIDX[0]                   |  <- propID for
+*            +-------------------------------+
+*            |  Array Pointer[0]             |  <- seek point of root node for this prop
+*            +-------------------------------+
+*            |  PropIDX[1]                   |
+*            +-------------------------------+
+*            |  Array Pointer[1]             |
+*            +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
+*            |  PropIDX[n_props-1]           |  up to number of props defined in
+*            +-------------------------------+  the index.  Not all will be filled
+*            |  Array Pointer[n_props-1]     |  since not all props are pre-sorted.
+*            +-------------------------------+  They are filled sequentially.
+*            |  Data for all arrays follows  |
+*            |  here.  (true?)               |
+*            +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
+*
+*  I (moseley) don't really follow why  DB->presorted_array and
+*   DB->presorted_root_node are needed during write operations.
+*
+*  Also, since the above is total number of props long, why not just
+*  have it indexed by propIDX (not propID)?  Then don't need to search
+*  for a given propID -- just do
+*
+*           sorted_props_index = read_sorted_props_table_from_disk();
+*           root_array_seek_pointer = sorted_props_index[ propIDX ];
+*
+*  Since the table above is fixed length, don't really need a loop to write
+*  or read from the disk.
+*
+*  Maybe the DB->presorted_(propid|root_node|array) could all be part of
+*  one structure (i.e. DB->presorted[n].array ).
+*
+*  These functions return an int, but never seems to be used.
+*
+*  Another problem with using this code is that the format of the array is
+*  different before and after writing to disk.  That is, with the old code
+*  pre_sort.c: CreatePropertySortArray() created an integer array total_files long.
+*  And when reading the the array back from disk you get that same array.
+*  This problem shows up in merge.c: load_filename_sort().  See notes there.
+*
+*
+*
+*******************************************************************************/
+
+
 int     DB_InitWriteSortedIndex_Native(void *db, int n_props)
 {
    struct Handle_DBNative *DB = (struct Handle_DBNative *) db;
@@ -1669,16 +1752,19 @@ int     DB_InitWriteSortedIndex_Native(void *db, int n_props)
 
    DB->offsets[SORTEDINDEX] = sw_ftell(fp);
 
-   /* Write number of properties */
+   /* Write number of properties possible.  May not write all*/
    printlong(fp,(unsigned long) n_props, sw_fwrite);
 
    DB->n_presorted_array = n_props;
    DB->presorted_array = (ARRAY **)emalloc(n_props * sizeof(ARRAY *));
    DB->presorted_root_node = (unsigned long *)emalloc(n_props * sizeof(unsigned long));
+
+
    for(i = 0; i < n_props ; i++)
    {
        DB->presorted_array[i] = NULL;
        DB->presorted_root_node[i] = 0;
+
        /* Reserve space for propidx and Array Pointer */
        printfileoffset(fp,(sw_off_t) 0, sw_fwrite);
        printfileoffset(fp,(sw_off_t) 0, sw_fwrite);
@@ -1687,31 +1773,55 @@ int     DB_InitWriteSortedIndex_Native(void *db, int n_props)
    return 0;
 }
 
-int     DB_WriteSortedIndex_Native(int propID, unsigned char *data, int sz_data, void *db)
+
+/***********************************************************************************
+* DB_WriteSortedIndex -
+*
+* Jose's ARRAY storage - a way to avoid reading the entire table into memory
+*
+* Input:
+*   propID      id of property to write
+*   *data       pointer to array of integers (total_files long)
+*   sz_data     size of array (i.e. total_files)
+*   *db         database
+*
+* Why pass *char and then cast to an *int?  *int is the
+* original data type in pre_sort.c.  Why not "int *data"?
+*
+*
+* Questions:
+*   Why does DB->presorted_root_node[] array needed to be populated during the write
+*   operation?
+*
+*
+*
+************************************************************************************/
+
+
+int     DB_WriteSortedIndex_Native(int propID, int *data, int num_elements, void *db)
 {
    struct Handle_DBNative *DB = (struct Handle_DBNative *) db;
    FILE *fp = DB->fp_presorted;
    ARRAY *arr;
    int i;
-   int *num = (int *)data;
-   int n = sz_data/sizeof(int);
 
-   arr = ARRAY_Create(fp);
-   for(i = 0 ; i < n ; i++)
-   {
-       ARRAY_Put(arr,i,num[i]);
-/*
-       if(!(i%10000))
-       {
-            ARRAY_FlushCache(arr);
-            printf("%d %d            \r",propID,i);
-       }
-*/
-   }
+   arr = ARRAY_Create(fp);          /* Initialize the ARRAY storage */
 
+
+   /* Add all elemnts of array to the array */
+   for(i = 0 ; i < num_elements ; i++)
+       ARRAY_Put(arr,i,data[i]);
+
+
+
+   /* ARRAY_Close() returns the offset of the root node for this array */
    DB->presorted_root_node[DB->next_sortedindex] = ARRAY_Close(arr);
 
+
+   /* Seek to start of this record in the index */
    sw_fseek(fp,DB->offsets[SORTEDINDEX] + (1 + 2 * DB->next_sortedindex) * sizeof(unsigned long),SEEK_SET);
+
+   /* Write out the propID and the seek position of this array's root node */
    printlong(fp,(unsigned long) propID, sw_fwrite);
    printlong(fp,(unsigned long) DB->presorted_root_node[DB->next_sortedindex], sw_fwrite);
 
@@ -1720,12 +1830,23 @@ int     DB_WriteSortedIndex_Native(int propID, unsigned char *data, int sz_data,
    return 0;
 }
 
+
 int     DB_EndWriteSortedIndex_Native(void *db)
 {
    return 0;
 }
 
- 
+/********************************************************************************
+*  BTREE DB_InitReadSortedIndex
+*
+*  Reads the index table into memory:
+*       DB->presorted_array[]       all set to null
+*       DB->presorted_propid[]      list of propIDs
+*       DB->presorted_root_node[]   seek points for the array for each propID
+*
+*********************************************************************************/
+
+
 int     DB_InitReadSortedIndex_Native(void *db)
 {
    struct Handle_DBNative *DB = (struct Handle_DBNative *) db;
@@ -1737,9 +1858,15 @@ int     DB_InitReadSortedIndex_Native(void *db)
    /* Read number of properties */
    DB->n_presorted_array = readlong(fp,sw_fread);
 
+   /* init the arrays */
+
    DB->presorted_array = (ARRAY **)emalloc(DB->n_presorted_array * sizeof(ARRAY *));
    DB->presorted_root_node = (unsigned long *)emalloc(DB->n_presorted_array * sizeof(unsigned long));
    DB->presorted_propid = (unsigned long *)emalloc(DB->n_presorted_array * sizeof(unsigned long));
+
+
+   /* read the table into memory */
+
    for(i = 0; i < DB->n_presorted_array ; i++)
    {
        DB->presorted_array[i] = NULL;
@@ -1750,14 +1877,26 @@ int     DB_InitReadSortedIndex_Native(void *db)
 
 }
 
+
+
 int     DB_ReadSortedIndex_Native(int propID, unsigned char **data, int *sz_data,void *db)
 {
    struct Handle_DBNative *DB = (struct Handle_DBNative *) db;
    FILE *fp = DB->fp_presorted;
    int i;
 
+   /* Open the array for the propID specified, if not already open */
+
+
+   /* $$$ looks like a bug -- If cur_presorted_array is set, but the propid doesn't match
+    * Then it will search for the new matching propID.  But if it doesn't find it then
+    * DB-cur_presorted_array is STILL true, so it will return the wrong array.
+    * Seems like need to set DB->cur_presorted_array = NULL before entering the loop. */
+
    if(!DB->cur_presorted_array || DB->cur_presorted_propid != (unsigned long)propID)
    {
+       DB->cur_presorted_array = NULL;  /* Added to fix bug mentioned above - moseley */
+
        for(i = 0; i < DB->n_presorted_array ; i++)
        {
            if((unsigned long)propID == DB->presorted_propid[i])
@@ -1768,10 +1907,12 @@ int     DB_ReadSortedIndex_Native(int propID, unsigned char **data, int *sz_data
            }
        }
    }
+
+   /* If found, then return */
    if(DB->cur_presorted_array)
    {
        *data = (unsigned char *)DB->cur_presorted_array;
-       *sz_data = sizeof(DB->cur_presorted_array);
+       *sz_data = sizeof(DB->cur_presorted_array);  /* is this just a non-zero value to flag that something was returned? */
    }
    else
    {
@@ -1782,7 +1923,6 @@ int     DB_ReadSortedIndex_Native(int propID, unsigned char **data, int *sz_data
    return 0;
 }
 
-
 int     DB_ReadSortedData_Native(int *data,int index, int *value, void *db)
 {
     *value = ARRAY_Get((ARRAY *)data,index);
@@ -1794,7 +1934,48 @@ int     DB_EndReadSortedIndex_Native(void *db)
    return 0;
 }
 
-#else
+#else /* NOT USE_BTREE */
+
+/********************************************************************************
+*
+*   DB->offsets[SORTEDINDEX] => first record in table There's a record for each
+*   property that was sorted (not all are sorted) The data passed in has
+*   already been compressed
+*
+*                 +------------------------------+
+*                 | Pointer to next table entry  | <<-- DB->last_sortedindex
+*                 |   (initially zero)           |
+*                 +------------------------------+
+*                 | PropID                       |
+*                 +------------------------------+
+*                 | Data Length in bytes         |
+*                 +------------------------------+
+*                 | Data [....]                  |
+*                 +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
+*                 |                              | <<-- DB_next_sortedindex
+*                 +------------------------------+
+*
+*   Again, DB->offsets[SORTEDINDEX] points to the very first record in the
+*   index While writing, DB->next_sortedindex points to the next place to start
+*   writing.  DB->last_sortedindex points to the (initially zero) point to the
+*   next table.  So, when it is not null then last_sortedinex is used to update
+*   the pointer in the previous table to point to the next table.  A zero entry
+*   indicates that there are no more records.
+*
+* DB_InitWriteSortedIndex_Native should probably write a null for the first
+* record's "next table entry" and set next_ and last_ pointers.  Then
+* DB_EndWriteSortedIndex_Native call would not be needed.
+*
+* Notes/Questions:
+*   Seems like result_sort.c is the only place that loads this -- in LoadSortedProps.
+*   LoadSortedProps is used by merge, proplimit, and result_sort.  LoadSortedProps
+*   uncompresses the array (and pre_sort.c compresses).  Why isn't that compression
+*   and uncompression done here?
+*
+*
+********************************************************************************/
+
+
 int     DB_InitWriteSortedIndex_Native(void *db)
 {
    struct Handle_DBNative *DB = (struct Handle_DBNative *) db;
@@ -1804,6 +1985,17 @@ int     DB_InitWriteSortedIndex_Native(void *db)
    return 0;
 }
 
+/********************************************************************************
+* DB_WriteSortedIndex
+*
+* Input:
+*   propID      property id of this table
+*   *data       pointer to char array compressed table
+*   sz_data     size of the char array in bytes
+*   db          where to write
+*
+*********************************************************************************/
+
 int     DB_WriteSortedIndex_Native(int propID, unsigned char *data, int sz_data,void *db)
 {
    sw_off_t tmp1,tmp2;
@@ -1813,7 +2005,7 @@ int     DB_WriteSortedIndex_Native(int propID, unsigned char *data, int sz_data,
 
    sw_fseek(fp, DB->next_sortedindex, SEEK_SET);
 
-
+    /* save start of this record so can update previous record's "next record" pointer */
    tmp1 = sw_ftell(fp);
 
    printfileoffset(fp,(sw_off_t)0,sw_fwrite);  /* Pointer to next table if any */
@@ -1856,12 +2048,24 @@ int     DB_EndWriteSortedIndex_Native(void *db)
    return 0;
 }
 
- 
+
+/* Non Btree read functions */
+
+
 int     DB_InitReadSortedIndex_Native(void *db)
 {
    return 0;
 }
 
+/***********************************************************************************
+*  DB_ReadSortedIndex_Native -
+*
+*  Searches through the sorted indexes looking for one that matches the propID
+*  passed in.  If found then malloc's a table and reads it in.
+*  The data is in compressed format and is not uncompressed here
+*
+***********************************************************************************/
+
 int     DB_ReadSortedIndex_Native(int propID, unsigned char **data, int *sz_data,void *db)
 {
    sw_off_t next;
@@ -1869,17 +2073,21 @@ int     DB_ReadSortedIndex_Native(int propID, unsigned char **data, int *sz_data
    struct Handle_DBNative *DB = (struct Handle_DBNative *) db;
    FILE *fp = DB->fp;
 
+
+   /* seek to the first record */
    sw_fseek(fp,DB->offsets[SORTEDINDEX],SEEK_SET);
 
 
+   /* get seek position of the next record, if needed */
    next = readfileoffset(fp,sw_fread);
-        /* read propID */
+
+   /* read propID for this record */
    id = uncompress1(fp,sw_fgetc);
 
 
    while(1)
    {
-       if(id == propID)
+       if(id == propID)  /* this is the property we are looking for */
        {
            tmp = uncompress1(fp,sw_fgetc);
            *sz_data = tmp;
@@ -1921,7 +2129,7 @@ int     DB_EndReadSortedIndex_Native(void *db)
 
 
 
-/* 
+/*
 ** Jose Ruiz 04/00
 ** Store a portable long in a portable format
 */
@@ -1934,7 +2142,7 @@ void    printlong(FILE * fp, unsigned long num, size_t(*f_write) (const void *,
         progerrno("Error writing %d of %d bytes: ", sizeof(long), written );
 }
 
-/* 
+/*
 ** Jose Ruiz 04/00
 ** Read a portable long from a portable format
 */
@@ -1946,8 +2154,8 @@ unsigned long readlong(FILE * fp, size_t(*f_read) (void *, size_t, size_t, FILE
     return UNPACKLONG(num);     /* Make the number readable */
 }
 
-/* 
-** 2003/10 Jose Ruiz 
+/*
+** 2003/10 Jose Ruiz
 ** Store a file offset in a portable format
 */
 void    printfileoffset(FILE * fp, sw_off_t num, size_t(*f_write) (const void *, size_t, size_t, FILE *))
@@ -1959,8 +2167,8 @@ void    printfileoffset(FILE * fp, sw_off_t num, size_t(*f_write) (const void *,
         progerrno("Error writing %d of %d bytes: ", sizeof(num), written );
 }
 
-/* 
-** 2003/10Jose Ruiz 
+/*
+** 2003/10Jose Ruiz
 ** Read a file offset from a portable format
 */
 sw_off_t readfileoffset(FILE * fp, size_t(*f_read) (void *, size_t, size_t, FILE *))
@@ -1979,7 +2187,7 @@ sw_off_t readfileoffset(FILE * fp, size_t(*f_read) (void *, size_t, size_t, FILE
 *   Properties are written sequentially to the .prop file.
 *   Fixed length records of the seek position into the
 *   property file are written sequentially to the main index (which is why
-*   there's a separate .prop file).  
+*   there's a separate .prop file).
 *
 *   DB_InitWriteProperties is called first time a property is written
 *   to save the offset of the property index table in the main index.
@@ -2058,7 +2266,7 @@ void    DB_WriteProperty_Native( IndexFILE *indexf, FileRec *fi, int propID, cha
 
     if ( count <= 0 )
         return;
-    
+
 
     if (!DB->prop)
         progerr("Property database file not opened\n");
@@ -2097,7 +2305,7 @@ void    DB_WriteProperty_Native( IndexFILE *indexf, FileRec *fi, int propID, cha
 
     /* Second write the number of saved bytes */
     if( !uncompressed_len )   /* No compression */
-        saved_bytes = 0;    
+        saved_bytes = 0;
     else
         saved_bytes = uncompressed_len - buf_len;
     /* Write them */
@@ -2106,7 +2314,7 @@ void    DB_WriteProperty_Native( IndexFILE *indexf, FileRec *fi, int propID, cha
 #ifdef DEBUG_PROP
     prop_start_pos = sw_ftell(DB->prop);
 #endif
-    
+
 
 
     if ( (int)(written_bytes = sw_fwrite(buffer, 1, buf_len, DB->prop)) != buf_len) /* Write data */
@@ -2187,7 +2395,7 @@ void DB_WritePropPositions_Native(IndexFILE *indexf, FileRec *fi, void *db)
 #ifdef DEBUG_PROP
         start_seek = sw_ftell( DB->fp );
 #endif
-        
+
         /* Write in portable format */
         printfileoffset( DB->fp, prop_loc->seek, sw_fwrite );
 
@@ -2223,7 +2431,7 @@ void DB_ReadPropPositions_Native(IndexFILE *indexf, FileRec *fi, void *db)
 
     if ( count <= 0 )
         return;
-    
+
 
     /* create a place to store them */
 
@@ -2235,7 +2443,7 @@ void DB_ReadPropPositions_Native(IndexFILE *indexf, FileRec *fi, void *db)
 
 #ifndef USE_BTREE
     /* now calculate seek_pos */
-    /* printlong currently always writes sizeof(long) bytes (usually 4 bytes 
+    /* printlong currently always writes sizeof(long) bytes (usually 4 bytes
     ** for 32 bit architectures and 8 bytes for 64bit ones, so 4 or 8 bytes
     ** bytes are need for seek
     */
@@ -2259,7 +2467,7 @@ void DB_ReadPropPositions_Native(IndexFILE *indexf, FileRec *fi, void *db)
 #ifdef DEBUG_PROP
         sw_off_t    seek_start = sw_ftell( DB->fp );
 #endif
-        
+
         /* make an alias */
         PROP_LOCATION *prop_loc = &pindex->prop_position[ i ];
 
@@ -2286,7 +2494,7 @@ void DB_ReadPropPositions_Native(IndexFILE *indexf, FileRec *fi, void *db)
 #endif
 }
 
-    
+
 
 /****************************************************************************
 *   Reads a property from the property file
@@ -2332,7 +2540,7 @@ char   *DB_ReadProperty_Native(IndexFILE *indexf, FileRec *fi, int propID, int *
     prop_loc = &pindex->prop_position[ propIDX ];
 
     seek_pos = pindex->prop_position[propIDX].seek;
-    
+
     /* Any for this metaID? */
     if (!seek_pos )
     {
