@@ -1,18 +1,31 @@
 /*
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+$Id: db_native.h,v 1.45 2005/05/12 15:41:05 karman Exp $
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 18:19:34 CDT 2005
+** added GPL
+
+
 **
 **
 **
@@ -205,24 +218,30 @@ int     DB_EndReadWords_Native(void *db);
 
 
 int     DB_WriteFileNum_Native(int filenum, unsigned char *filedata,int sz_filedata, void *db);
-int     DB_ReadFileNum_Native(int *filenum, unsigned char *filedata,int sz_filedata, void *db);
+int     DB_ReadFileNum_Native( unsigned char *filedata, void *db);
 int     DB_CheckFileNum_Native(int filenum, void *db);
 int     DB_RemoveFileNum_Native(int filenum, void *db);
 
-#ifdef USE_BTREE
+/** Pre-sorted array access **/
+
+#ifdef USE_PRESORT_ARRAY
 int     DB_InitWriteSortedIndex_Native(void *db , int n_props);
-int     DB_WriteSortedIndex_Native(int propID, unsigned char *data, int sz_data,void *db);
+int     DB_WriteSortedIndex_Native(int propID, int *data, int num_elements,void *db);
 #else
 int     DB_InitWriteSortedIndex_Native(void *db );
 int     DB_WriteSortedIndex_Native(int propID, unsigned char *data, int sz_data,void *db);
 #endif
 int     DB_EndWriteSortedIndex_Native(void *db);
- 
+
 int     DB_InitReadSortedIndex_Native(void *db);
 int     DB_ReadSortedIndex_Native(int propID, unsigned char **data, int *sz_data,void *db);
+/* Note that a macro is not provided for reading the data */
 int     DB_ReadSortedData_Native(int *data,int index, int *value, void *db);
 int     DB_EndReadSortedIndex_Native(void *db);
 
+
+
+
 int     DB_InitWriteProperties_Native(void *db);
 void    DB_WriteProperty_Native( IndexFILE *indexf, FileRec *fi, int propID, char *buffer, int buf_len, int uncompressed_len, void *db);
 void    DB_WritePropPositions_Native(IndexFILE *indexf, FileRec *fi, void *db);
@@ -231,10 +250,10 @@ char   *DB_ReadProperty_Native(IndexFILE *indexf, FileRec *fi, int propID, int *
 void    DB_Reopen_PropertiesForRead_Native(void *db);
 
 #ifdef USE_BTREE
-int	   DB_InitWriteTotalWordsPerFile_Native(SWISH *sw, void *DB);
+int        DB_InitWriteTotalWordsPerFile_Native(SWISH *sw, void *DB);
 int    DB_WriteTotalWordsPerFile_Native(SWISH *sw, int idx, int wordcount, void *DB);
 int    DB_EndWriteTotalWordsPerFile_Native(SWISH *sw, void *DB);
-int	   DB_InitReadTotalWordsPerFile_Native(SWISH *sw, void *DB);
+int        DB_InitReadTotalWordsPerFile_Native(SWISH *sw, void *DB);
 int    DB_ReadTotalWordsPerFile_Native(SWISH *sw, int idx, int *wordcount, void *DB);
 int    DB_EndReadTotalWordsPerFile_Native(SWISH *sw, void *DB);
 #endif
