@@ -1,20 +1,30 @@
 /*
-**
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**
-**
+
+$Id: db_read.c,v 1.25 2005/05/12 15:41:05 karman Exp $
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
+
 **
 ** 2001-05-07 jmruiz init coding
 **
@@ -227,8 +237,14 @@ void    read_header(SWISH *sw, INDEXDATAHEADER *header, void *DB)
             header->totalwords = tmp1;
             header->totalfiles = tmp2;
             header->total_word_positions = tmp3;
-	    header->removedfiles = tmp4;
+            header->removedfiles = tmp4;
             break;
+
+        case TOTALWORDS_REMOVED_ID:  /* Added here instead of above to keep index compatible */
+            parse_int_from_buffer(tmp, buffer);
+            header->removed_word_positions = tmp;
+            break;
+
 /* removed due to patents problems
         case FILEINFOCOMPRESSION_ID:
             ReadHeaderInt(itmp, fp);
@@ -262,7 +278,7 @@ void    read_header(SWISH *sw, INDEXDATAHEADER *header, void *DB)
 #endif
 
         default:
-            progerr("Severe index error in header");
+            progerr("Severe index error in header.  Unknown index header ID: %d", id );
             break;
         }
         efree(buffer);
@@ -367,24 +383,23 @@ void parse_integer_table_from_buffer(int table[], int table_size, char *buffer)
 
 /* Used by rank.c */
 
+int getTotalWordsInFile( IndexFILE *indexf, int filenum )
+{
+    if ( filenum < 1 || filenum > indexf->header.totalfiles )
+        progerr("getTotalWordsInFile passed an invalied file number");
 
+    /* This is still one too many layers */
 #ifdef USE_BTREE
-void getTotalWordsPerFile(SWISH *sw, IndexFILE *indexf, int idx,int *wordcount)
-{
-        DB_ReadTotalWordsPerFile(sw, idx, wordcount, indexf->DB);
-}
-
+    return DB_CheckFileNum( indexf->sw, filenum, indexf->DB );
 #else
-
-
-void getTotalWordsPerFile(IndexFILE *indexf, int idx,int *wordcount)
-{
-    INDEXDATAHEADER *header = &indexf->header;
-
-    *wordcount = header->TotalWordsPerFile[idx];
-}
+    if ( indexf->header.ignoreTotalWordCountWhenRanking )
+        progerr("Can't return total words -- index was not built with IgnoreTotalWordCountWhenRanking");
+    else
+        return indexf->header.TotalWordsPerFile[filenum - 1];
 
 #endif
+    return 0;  /* make the compiler quiet */
+}
 
 /*------------------------------------------------------*/
 /*---------- General entry point of DB module ----------*/
@@ -454,9 +469,9 @@ int     DB_CheckFileNum(SWISH *sw, int filenum, void *DB)
    return sw->Db->DB_CheckFileNum(filenum, DB);
 }
 
-int     DB_ReadFileNum(SWISH *sw, int *filenum, unsigned char *filedata,int sz_filedata, void *DB)
+int     DB_ReadFileNum(SWISH *sw, unsigned char *filedata, void *DB)
 {
-   return sw->Db->DB_ReadFileNum(filenum, filedata,sz_filedata, DB);
+   return sw->Db->DB_ReadFileNum( filedata, DB);
 }
 
  
@@ -470,10 +485,12 @@ int     DB_ReadSortedIndex(SWISH *sw, int propID, unsigned char **data, int *sz_
    return sw->Db->DB_ReadSortedIndex(propID, data, sz_data,DB);
 }
 
+/* ******* This is now a macro and accessies the native data by default
 int     DB_ReadSortedData(SWISH *sw, int *data,int index, int *value, void *DB)
 {
    return sw->Db->DB_ReadSortedData(data,index,value,DB);
 }
+*******/
 
 int     DB_EndReadSortedIndex(SWISH *sw, void *DB)
 {
