@@ -1,20 +1,30 @@
 /*
-**
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**
-**
+
+$Id: db_write.c,v 1.29 2005/09/06 18:10:34 whmoseley Exp $
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
 **
 ** 2001-05-07 jmruiz init coding
 **
@@ -75,12 +85,17 @@ static int    write_hash_words_to_header(SWISH *sw, int header_ID, struct swline
         DB_WriteHeaderData((sw),(id), (unsigned char *)itmp, sizeof(long) * 4, (DB)); \
 }
 
-
-void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filename, int totalwords, int totalfiles, int total_word_positions, int removedfiles, int merged)
+void    write_header(SWISH *sw, int merged_flag )
 {
-    char   *c,
-           *tmp;
+    IndexFILE       *indexf     = sw->indexlist;       /* first element in the list */
+    INDEXDATAHEADER *header     = &indexf->header;
+    char            *filename   = indexf->line;
+    void            *DB         = indexf->DB;
+    char            *c;
+    char            *tmp;
 
+
+    /* $$$ this isn't portable */
     c = (char *) strrchr(filename, '/');
     if (!c || (c && !*(c + 1)))
         c = filename;
@@ -91,10 +106,16 @@ void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filen
 
     DB_WriteHeaderData(sw, INDEXHEADER_ID, (unsigned char *)INDEXHEADER, strlen(INDEXHEADER) +1, DB);
     DB_WriteHeaderData(sw, INDEXVERSION_ID, (unsigned char *)INDEXVERSION, strlen(INDEXVERSION) + 1, DB);
-    write_header_int(sw, MERGED_ID, merged, DB);
+    write_header_int(sw, MERGED_ID, merged_flag, DB);
     DB_WriteHeaderData(sw, NAMEHEADER_ID, (unsigned char *)header->indexn, strlen(header->indexn) + 1, DB);
     DB_WriteHeaderData(sw, SAVEDASHEADER_ID, (unsigned char *)c, strlen(c) + 1, DB);
-    write_header_int4(sw, COUNTSHEADER_ID, totalwords, totalfiles, total_word_positions, removedfiles, DB);
+
+    write_header_int4(sw, COUNTSHEADER_ID,
+            header->totalwords,
+            header->totalfiles,
+            header->total_word_positions + indexf->total_word_positions_cur_run, /* Total this run's total words (not unique) with any previous */
+            header->removedfiles, DB);
+
     tmp = getTheDateISO();
     DB_WriteHeaderData(sw, INDEXEDONHEADER_ID, (unsigned char *)tmp, strlen(tmp) + 1,DB);
     efree(tmp);
@@ -137,9 +158,11 @@ void    write_header(SWISH *sw, INDEXDATAHEADER * header, void * DB, char *filen
 #ifndef USE_BTREE
     /* Write the total words per file array, if used */
     if ( !header->ignoreTotalWordCountWhenRanking )
-        write_integer_table_to_header(sw, TOTALWORDSPERFILE_ID, header->TotalWordsPerFile, totalfiles, DB);
+        write_integer_table_to_header(sw, TOTALWORDSPERFILE_ID, header->TotalWordsPerFile, header->totalfiles, DB);
 #endif
 
+    write_header_int(sw, TOTALWORDS_REMOVED_ID, header->removed_word_positions, DB);
+
     DB_EndWriteHeader(sw, DB);
 }
 
@@ -597,6 +620,7 @@ unsigned char *q;
             p2+= sizeof(long);
             curmetanamepos_2= p2 - newdata;
         }
+        PACKLONG2(p - sw->Index->worddata_buffer, sw->Index->worddata_buffer + curmetanamepos);
     }
 
 
@@ -865,21 +889,29 @@ int     DB_RemoveFileNum(SWISH *sw, int filenum, void *DB)
 }
 
 
-#ifdef USE_BTREE
+#ifdef USE_PRESORT_ARRAY
 int     DB_InitWriteSortedIndex(SWISH *sw, void *DB, int n_props)
 {
    return sw->Db->DB_InitWriteSortedIndex(DB, n_props);
 }
+
+int     DB_WriteSortedIndex(SWISH *sw, int propID, int *data, int sz_data,void *DB)
+{
+   return sw->Db->DB_WriteSortedIndex(propID, data, sz_data,DB);
+}
+
+
 #else
 int     DB_InitWriteSortedIndex(SWISH *sw, void *DB)
 {
    return sw->Db->DB_InitWriteSortedIndex(DB);
 }
-#endif
+
 int     DB_WriteSortedIndex(SWISH *sw, int propID, unsigned char *data, int sz_data,void *DB)
 {
    return sw->Db->DB_WriteSortedIndex(propID, data, sz_data,DB);
 }
+#endif
 
 int     DB_EndWriteSortedIndex(SWISH *sw, void *DB)
 {
