@@ -1,6 +1,31 @@
 /*
-$Id: double_metaphone.c,v 1.3 2004/03/09 20:13:24 whmoseley Exp $
+$Id: double_metaphone.c,v 1.6 2005/05/12 15:41:05 karman Exp $
 **
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
+
 **
 ** August 20, 2002 moseley - first added to swish-e
 **
@@ -31,6 +56,10 @@ COPYRIGHT
   modify it under the same terms as Perl itself.
 
 
+** Mon May  9 16:19:47 CDT 2005
+** "the same terms as Perl itself" means GPL if we want it. We do.
+**
+
 **
 **
 */
