@@ -1,23 +1,38 @@
 /*
-$Id: entities.c,v 1.18 2004/03/09 20:13:24 whmoseley Exp $
-**
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+$Id: entities.c,v 1.22 2006/10/18 02:58:02 karman Exp $
 **
 ** (c) Rainer.Scherg
 **
 **
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
+**************************************************************************************
+
+
+
+
 ** HTML entity routines (encoding, etc.):
 **
 ** internally we are working with int/wchar_t to support unicode-16 for future
@@ -632,9 +647,7 @@ unsigned char *strConvHTMLEntities2ISO(unsigned char *buf)
 
 int     charEntityDecode(unsigned char *s, unsigned char **end)
 {
-    unsigned char *s1,
-           *t;
-    char *e_end;
+    unsigned char *s1, *t, *e_end;
     unsigned char s_cmp[MAX_ENTITY_LEN + 1];
     int     len;
     int     code;
