@@ -1,22 +1,35 @@
 /*
-$Id: file.c,v 1.52 2003/07/21 18:16:12 whmoseley Exp $
+$Id: file.c,v 1.59 2006/06/11 02:28:19 augur Exp $
 **
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
+
 **-------------------------------------------------------------
 ** Changed getdefaults to allow metaNames in the user
 ** configuration file
@@ -72,7 +85,7 @@ $Id: file.c,v 1.52 2003/07/21 18:16:12 whmoseley Exp $
 #endif
 
 /* Cough, hack, cough - convert slash to backslash for programs that are run via the shell */
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(__CYGWIN__)
 void make_windows_path( char *path )
 {
     char    *c;
@@ -88,7 +101,7 @@ void make_windows_path( char *path )
 char *
 get_libexec(void){
 	char *fn;
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(__CYGWIN__)
 	char *tr;
 	int pos;
 
@@ -145,8 +158,19 @@ void normalize_path(char *path)
 
     while( len > 1 && path[len-1] == '/' )
     {
+#if defined(_WIN32) && !defined(__CYGWIN__)
+        /* c:/ must end with / but other directories must not */
+        if( path[1] == ':' && len == 3 ){
+            break;
+        }
+        else {
+            path[len-1] = '\0';
+            len--;
+        }
+#else
         path[len-1] = '\0';
         len--;
+#endif
     }
 }
 
@@ -250,7 +274,7 @@ char   *read_stream(SWISH *sw, FileProp *fprop, int is_text)
         *buffer = '\0';
 
         bytes_read = fread(buffer, 1, filelen, fprop->fp);
-        buffer[bytes_read+1] = '\0';  /* hopfully doesn't read more than filelen bytes ;) */
+        buffer[bytes_read] = '\0';  /* hopfully doesn't read more than filelen bytes ;) */
 
         
 
@@ -285,10 +309,23 @@ char   *read_stream(SWISH *sw, FileProp *fprop, int is_text)
 
     /* filelen was zero so we are reading from a handle */
 
+    /* 
+     * No, if filelen is zero and fprop->hasfilter is set, then we are
+     * reading from a filter and need to read the entire stream in.
+     * This broke when using -S prog and a zero length file came along. - moseley Mar 2005
+     */
+
     bufferlen = RD_BUFFER_SIZE;
     buffer = (unsigned char *)Mem_ZoneAlloc(sw->Index->perDocTmpZone, bufferlen + 1);
     *buffer = '\0';
 
+
+    /* catches case where source file is zero bytes */
+    /* but filter may still produce outut */
+    if ( !fprop->hasfilter )
+        return (char *)buffer;
+
+
     offset = 0;
     while ( 1 )
     {
