@@ -1,19 +1,30 @@
 /*
-** $Id: headers.c,v 1.10 2004/11/09 23:03:53 karman Exp $
+** $Id: headers.c,v 1.17 2005/05/12 15:41:05 karman Exp $
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
 */
 
 #include "swish.h"
@@ -54,9 +65,12 @@ typedef struct
 static HEADER_MAP header_map[] = {
     {  "Name",              SWISH_STRING, 2,  offsetof( INDEXDATAHEADER, indexn ) },
     {  "Saved as",          SWISH_STRING, 2,  offsetof( INDEXDATAHEADER, savedasheader ) },
+    /* $$$ Total Words is  unique words, and is Not corrected for removed words */
     {  "Total Words",       SWISH_NUMBER, 2,  offsetof( INDEXDATAHEADER, totalwords ) },
     {  "Total Files",       SWISH_NUMBER, 2,  offsetof( INDEXDATAHEADER, totalfiles ) },
+    {  "Removed Files",     SWISH_NUMBER, 2,  offsetof( INDEXDATAHEADER, removedfiles ) },
     {  "Total Word Pos",    SWISH_NUMBER, 2,  offsetof( INDEXDATAHEADER, total_word_positions ) },
+    {  "Removed Word Pos",  SWISH_NUMBER, 2,  offsetof( INDEXDATAHEADER, removed_word_positions ) },
     {  "Indexed on",        SWISH_STRING, 2,  offsetof( INDEXDATAHEADER, indexedon ) },
     {  "Description",       SWISH_STRING, 2,  offsetof( INDEXDATAHEADER, indexd ) },
     {  "Pointer",           SWISH_STRING, 2,  offsetof( INDEXDATAHEADER, indexp ) },
@@ -250,8 +264,9 @@ SWISH_HEADER_VALUE SwishHeaderValue( SWISH *sw, const char *index_name, const  c
 
 static SWISH_HEADER_VALUE fetch_single_header( IndexFILE *indexf, HEADER_MAP *header_map, SWISH_HEADER_TYPE *data_type )
 {
-    SWISH_HEADER_VALUE value;
-    INDEXDATAHEADER    *header = &indexf->header;
+    SWISH_HEADER_VALUE  value;
+    INDEXDATAHEADER     *header = &indexf->header;
+    char                *data_pointer = (char *)header + header_map->offset; /* should that be void* as a generic address? */
 
     value.string = NULL;
 
@@ -260,24 +275,33 @@ static SWISH_HEADER_VALUE fetch_single_header( IndexFILE *indexf, HEADER_MAP *he
     switch ( header_map->data_type )
     {
         case SWISH_STRING:
-            value.string = *(const char **) ( (char *)header + header_map->offset );
+            value.string = *(const char **) data_pointer;
             return value;
 
         case SWISH_NUMBER:
         case SWISH_BOOL:
-            value.number = *(unsigned long *) ( (char *)header + header_map->offset );
+            value.number = *(unsigned long *) data_pointer;
+
+            /* $$$ Ugly hack alert! */
+            /* correct for removed files */
+            if ( (void *)data_pointer == &header->totalfiles )
+                value.number -= header->removedfiles;
+
+            if ( (void *)data_pointer == &header->total_word_positions )
+                value.number -= header->removed_word_positions;
+
             return value;
 
         case SWISH_LIST:
         {
-            struct swline *first_item = *(struct swline **)( (char *)header + header_map->offset );
+            struct swline *first_item = *(struct swline **) data_pointer;
             value.string_list = create_string_list( indexf->sw, first_item );
             return value;
         }
 
         case SWISH_WORD_HASH:
         {
-            WORD_HASH_TABLE table = *(WORD_HASH_TABLE *)( (char *)header + header_map->offset );
+            WORD_HASH_TABLE table = *(WORD_HASH_TABLE *) data_pointer;
             *data_type = SWISH_LIST;
             value.string_list = string_list_from_hash( indexf->sw, table ); 
             return value;
