@@ -1,21 +1,33 @@
 /*
-$Id: index.c,v 1.232 2004/12/06 22:08:41 whmoseley Exp $
+$Id: index.c,v 1.249 2007/01/05 22:14:10 whmoseley Exp $
 **
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
-** This program and library is free software; you can redistribute it and/or
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-**  long with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:51:39 CDT 2005
+** added GPL
+
+
 **--------------------------------------------------------------------
 ** ** ** PATCHED 5/13/96, CJC
 **
@@ -141,9 +153,7 @@ $Id: index.c,v 1.232 2004/12/06 22:08:41 whmoseley Exp $
 static void index_path_parts( SWISH *sw, char *path, path_extract_list *list, INDEXDATAHEADER *header, docProperties **properties );
 static void SwapLocData(SWISH *,ENTRY *,unsigned char *,int);
 static void unSwapLocData(SWISH *,int, ENTRY *);
-#ifndef USE_BTREE
 static void sortSwapLocData(ENTRY *);
-#endif
 
 /* 
   -- init structures for this module
@@ -698,6 +708,220 @@ LOCATION *add_position_location(void *oldp, struct MOD_Index *idx, int frequency
         return newp;
 }
 
+#ifdef USE_BTREE
+/**********************************************************************
+* file_is_newer_than_existing 
+*       - returns true if the new file (based on fprop) is newer than the existing
+*
+* Call with:
+*   fprop - file prop for the new file
+*   sw    - current SWISH
+*   existing_number - file number
+*
+* Returns:
+*   true if new file is newer than existing and existing should be deleted
+*
+***********************************************************************/
+
+static int file_is_newer_than_existing( SWISH *sw,  FileProp * fprop, int existing_filenum)
+{
+    IndexFILE   *indexf = sw->indexlist;
+    int         ret;
+    propEntry   *existing_prop;
+    propEntry   *new_prop;
+    FileRec     fi;
+    int         error_flag;
+    unsigned long tmp;
+
+    /* Get the date from the existing file */
+
+
+    /* Fetch metaEntry */
+    if(!indexf->modified_meta)
+        indexf->modified_meta = getPropNameByName( &indexf->header, AUTOPROPERTY_LASTMODIFIED );
+
+
+    memset(&fi, 0, sizeof( FileRec ));
+    fi.filenum = existing_filenum;
+    existing_prop = ReadSingleDocPropertiesFromDisk(indexf, &fi, indexf->modified_meta->metaID, 0 );
+
+    /* Don't need the fi struct any more, but can't call freefileinfo() */
+    if ( fi.prop_index )
+        efree( fi.prop_index );
+
+
+
+    /* Create a property for the new file */
+
+    if ( !fprop->mtime ) /* was a time provided?  */
+        new_prop = NULL;  /* $$$ should be MISSING or NO_PROPERTY */
+
+    else /* create a property */
+    {
+        tmp = PACKLONG(fprop->mtime);
+        new_prop = CreateProperty( indexf->modified_meta, (unsigned char *)&tmp, sizeof( tmp ), 1, &error_flag );
+    }
+
+
+    /* -1 = existing < new_prop | +1 existing > new_prop */
+    ret = Compare_Properties( indexf->modified_meta, existing_prop, new_prop );
+
+
+    /* Skip new file is older or same age as existing file */
+    /* and at least one of the files has a date */
+    /* if neither file has a date then the new one will replace existing */
+
+    if (ret >= 0 && ( existing_prop || new_prop ) )
+    {
+        if (sw->verbose >= 3)
+        {
+           if ( new_prop )
+                printf(" - Update mode - File '%s' same or older than existing filenum: %d - (Skipping it)\n", fprop->real_path, existing_filenum);
+            else
+                printf(" - Update mode - File '%s' exists with date, but new files does not have a date.  Keeping existing file. filenum: %d\n", fprop->real_path, existing_filenum);
+        }
+
+        freeProperty( existing_prop );
+        freeProperty( new_prop );
+
+        return 0;
+    }
+
+
+
+    /* new file is newer so replace old file */
+
+    if (sw->verbose >= 3)
+        printf(" - Update mode - File '%s' replaced existing file number %d because %s\n",
+                    fprop->real_path,
+                    existing_filenum,
+                    existing_prop && new_prop
+                        ? "it is newer than the existing file"
+                        : existing_prop || new_prop
+                            ? "only the new file has a date"
+                            : "neither file has a date"
+         );
+
+    freeProperty( existing_prop );
+    freeProperty( new_prop );
+
+    return 1;  /* go ahead and remove */
+}
+#endif
+
+
+/***************************************************************************
+* check_for_replace -  Tests if replacing a file in update mode
+* Call with:
+*   SWISH       *sw     - the swish object
+*   FileProp    *fprop  - file properties
+*
+* Returns:
+*   true if ok to index new file
+*   false means skip indexing this file
+*
+****************************************************************************/
+static int check_for_replace( SWISH *sw, FileProp * fprop )
+{
+
+#ifndef USE_BTREE
+    return 1;
+#else
+
+    int         existing_filenum;
+    int         existing_is_deleted;
+    int         existing_word_count = 0;
+    IndexFILE   *indexf = sw->indexlist;
+    int         update_mode = sw->Index->update_mode;
+    char        *update_string;
+    int         return_value;
+    int         delete_existing = 0;  /* flag to delete existing file */
+
+    /* Return true if not in update or remove mode */
+    if ( MODE_UPDATE != update_mode && MODE_REMOVE != update_mode )
+        return 1;
+
+
+    update_string = MODE_UPDATE == update_mode ? "Update" : "Remove";
+
+    /* Always skip in remove mode.  In update, skip index if doesn't exist as below */
+
+    return_value  = MODE_UPDATE == update_mode ? 1 : 0;  /* how to exit based on mode */
+
+
+
+    existing_filenum = DB_ReadFileNum(sw, fprop->real_path, indexf->DB);
+
+    /* If the file already exists then likely will remove the file, so lookup the old word count */
+    /* Assumes that zero words indicates that a file was already deleted */
+    if ( existing_filenum )
+        existing_word_count = getTotalWordsInFile( indexf, existing_filenum );
+
+    existing_is_deleted = existing_filenum && ( 0 == existing_word_count);  /* just to make it clear */
+
+
+
+    if ( sw->verbose >= 5 )
+        printf("\nFile %s.  Existing filenum: %d.  Existing is deleted: %d Existing wordcount: %d\n",
+                fprop->real_path, existing_filenum, existing_is_deleted, existing_word_count );
+
+
+
+
+    /* Skip if file does not already exist or has not already been deleted */
+
+    if ( !existing_filenum )
+    {
+        if ( sw->verbose >= 4 )
+            printf(" - %s Mode - File '%s' is a new file and does not exist in the index\n", update_string, fprop->real_path );
+
+        return return_value;
+    }
+
+
+
+    if ( existing_is_deleted )
+    {
+        if ( sw->verbose >= 4 )
+            printf(" - %s Mode - Existing file '%s' has already been deleted from the index\n", update_string, fprop->real_path );
+
+        return return_value;
+
+    }
+
+
+    /* At this point we have an existing file */
+
+
+    /* MODE_REMOVE is always deleted.  In update mode, delete if current file is newer than existing */
+
+
+    delete_existing = MODE_REMOVE == update_mode
+                        ? 1
+                        : file_is_newer_than_existing( sw, fprop, existing_filenum  );
+
+
+    if ( delete_existing )
+    {
+
+        DB_RemoveFileNum(sw,existing_filenum,indexf->DB);
+        indexf->header.removedfiles++;
+        indexf->header.removed_word_positions += existing_word_count;
+
+        if ( sw->verbose >= 3 )
+            printf(" - %s Mode - Removed existing file '%s' (#%d) from index\n", update_string, fprop->real_path, existing_filenum );
+
+        return return_value;  /* keep indexing in update mode, other wise don't index */
+
+    }
+
+    return 0;  /* don't index the new file -- we are either keeping or are in remove mode */
+
+#endif
+
+}
+
+
 /***********************************************************************
    -- Start the real indexing process for a file.
    -- This routine will be called by the different indexing methods
@@ -719,9 +943,6 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
     char        strType[30];
     int         i;
     FileRec     fi;  /* place to hold doc properties */
-#ifdef USE_BTREE
-    int old_filenum;
-#endif
 
     memset( &fi, 0, sizeof( FileRec ) );
 
@@ -757,10 +978,8 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
     /* Get input file handle */
     if (fprop->hasfilter)
     {
-        fprop->fp = FilterOpen(fprop);
-
-        /* This should be checked in filteropen because the popen probably won't fail */
-        if ( !fprop->fp )
+        /* This checks for a non-null file handle -- but may return true with popen regardless */
+        if ( !FilterOpen(fprop) )
             progerr("Failed to open filter for file '%s'",fprop->real_path);
     }
 
@@ -788,78 +1007,25 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
         fprop->real_path = process_regex_list( fprop->real_path, sw->replaceRegexps, &matched );
     }
 
-#ifdef USE_BTREE
-    /** Let's see if file already exits when in update mode */
-    switch(sw->Index->update_mode)
+
+    /* Check for the need to remove an existing file first */
+    if ( !check_for_replace( sw, fprop ) )
     {
+        /* report file skipped if not in remove mode
+         * all files are skipped in remove mode */
 
-      case MODE_UPDATE:      /* Update mode */
+        if ( sw->verbose >= 1 && MODE_REMOVE != sw->Index->update_mode )
+            printf("Document '%s' not added to index.\n\n", fprop->real_path);
 
-        DB_ReadFileNum(sw,&old_filenum,fprop->real_path,strlen(fprop->real_path),indexf->DB);
-        /* If exits a previous file with the same real_path ... */
-        if(old_filenum)
-        {
-            IndexFILE   *cur_index = sw->indexlist;
-            FileRec     fi;
-            int         ret;
-            propEntry   *wp, *cp;
-            int         error_flag;
-            unsigned long tmp;
+        if (fprop->fp)
+            flush_stream( fprop );
 
-            /* Let's see if file is newer base on mtime property */
-            if(!cur_index->modified_meta)
-                cur_index->modified_meta = getPropNameByName( &cur_index->header, AUTOPROPERTY_LASTMODIFIED );
-
-            memset(&fi, 0, sizeof( FileRec ));
-            fi.filenum = old_filenum;
-            cp = ReadSingleDocPropertiesFromDisk(cur_index, &fi, cur_index->modified_meta->metaID, 0 );
-
-            /* Create a property based on mtime in order to use it
-            ** for comparing properties in Compare_Property routine */
-            tmp = PACKLONG(fprop->mtime);
-            wp = CreateProperty( cur_index->modified_meta, (unsigned char *)&tmp, sizeof( tmp ), 1, &error_flag );
-
-            ret = Compare_Properties( cur_index->modified_meta, cp, wp );
-
-            freeProperty( cp );     
-            freeProperty( wp );     
-            if ( fi.prop_index )
-                efree( fi.prop_index );
-
-            /* New file is the same or older. Skip it */
-            if (ret >= 0)
-            {
-               if (sw->verbose >= 3)
-                   printf(" - Update mode - File same or older - (Skipping it)\n\n");
-
-                return;
-            }
-            else
-            {  /* Remove old filenum and continue */
-                DB_RemoveFileNum(sw,old_filenum,indexf->DB);
-                cur_index->header.removedfiles++;
-            }
-        }
-        break;
-
-
-
-      case MODE_REMOVE:      /* Remove mode */
-
-        DB_ReadFileNum(sw,&old_filenum,fprop->real_path,strlen(fprop->real_path),indexf->DB);
-        /* If exits a previous file with the same real_path remove it */
-        if(old_filenum)
-        {
-            IndexFILE   *cur_index = sw->indexlist;
-            /* Remove old filenum and continue */
-            DB_RemoveFileNum(sw,old_filenum,indexf->DB);
-            cur_index->header.removedfiles++;
-        }
         return;
-        break;
     }
 
-#endif
+
+
+
 
     /** Read the buffer, if not a stream parser **/
 
@@ -954,7 +1120,7 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
 
     /* Now bump the file counter  */
     idx->filenum++;
-    indexf->header.totalfiles++; /* why ??? is this needed */
+    indexf->header.totalfiles++;
     fi.filenum = idx->filenum;
 
     /** PARSE **/
@@ -967,7 +1133,7 @@ void    do_index_file(SWISH * sw, FileProp * fprop)
     if (!fprop->external_program)  /* external_program is not set if a filter is in use */
     {
         if (fprop->hasfilter)
-            FilterClose(fprop->fp); /* close filter pipe - should the filter be flushed? */
+            FilterClose(fprop); /* close filter pipe - should the filter be flushed? */
         else
             sw_fclose(fprop->fp); /* close file */
     }
@@ -1129,7 +1295,7 @@ void   addentry(SWISH * sw, ENTRY *e, int filenum, int structure, int metaID, in
     struct MOD_Index *idx = sw->Index;
 
 
-    indexf->total_word_positions++;
+    indexf->total_word_positions_cur_run++;
 
     if ( DEBUG_MASK & DEBUG_WORDS )
     {
@@ -1943,7 +2109,6 @@ void    write_index(SWISH * sw, IndexFILE * indexf)
     if ( !(ep = sw->Index->entryArray ))
         return;  /* nothing to do */
 
-
     totalwords = ep->numWords;
 
     DB_InitWriteWords(sw, indexf->DB);
@@ -2104,6 +2269,8 @@ void    write_index(SWISH * sw, IndexFILE * indexf)
     ENTRY  *epi;
     int     totalwords;
     int     percent, lastPercent, n;
+    int     last_loc_swap;
+
     long    old_wordid;
     unsigned char *buffer =NULL;
     int     sz_buffer = 0;
@@ -2136,54 +2303,72 @@ void    write_index(SWISH * sw, IndexFILE * indexf)
     if(sw->Index->swap_locdata)
         Mem_ZoneReset(sw->Index->totalLocZone);
 
-    n = lastPercent = 0;
-    for (i = 0; i < totalwords; i++)
+    n = lastPercent = last_loc_swap = -1;
+    for (i = 0; i < VERYBIGHASHSIZE; i++)
     {
-        if ( sw->verbose && totalwords > 10000 )  // just some random guess
+         /* If we are in economic mode -e restore locations */
+        if(sw->Index->swap_locdata)
         {
-            n++;
-            percent = (n * 100)/totalwords;
-            if (percent - lastPercent >= DELTA )
+            if (((i * (MAX_LOC_SWAP_FILES - 1)) / (VERYBIGHASHSIZE - 1)) != last_loc_swap)
             {
-                printf("\r  Writing word text: %3d%%", percent );
-                fflush(stdout);
-                lastPercent = percent;
+                /* Free not longer needed memory */
+                Mem_ZoneReset(sw->Index->totalLocZone);
+                last_loc_swap = (i * (MAX_LOC_SWAP_FILES - 1)) / (VERYBIGHASHSIZE - 1);
+                unSwapLocData(sw, last_loc_swap, NULL );
             }
         }
-
-        epi = ep->elist[i];
-
-        /* why check for stopwords here?  removestopwords could have remove them */
-
-        if ( !is_word_in_hash_table( indexf->header.hashstoplist, epi->word ) )
+        if ((epi = sw->Index->hashentries[i]))
         {
-            /* Build worddata buffer */
-            build_worddata(sw, epi);
-            /* let's see if word is already in the index */
-            old_wordid = read_worddata(sw, epi, indexf, &buffer, &sz_buffer);
-            /* If exists, we have to add the new worddata buffer to the old one */
-            if(old_wordid)
+            while (epi)
             {
-                 add_worddata(sw, buffer, sz_buffer);
-                 efree(buffer);
-                 buffer = NULL;
-                 sz_buffer = 0;
-                 delete_worddata(sw, old_wordid, indexf);
-                 write_worddata(sw, epi, indexf);
-                 update_wordID(sw, epi, indexf);
-            }
-            else
-            {
-                 /* Reset last error. It was set in read_worddata if
-                 ** word was not found */
-                 sw->lasterror = RC_OK;
-                 /* Write word to index file */
-                 write_worddata(sw, epi, indexf);
-                 write_word(sw, epi, indexf);
+                /* If we are in economic mode -e we must sort locations by metaID, filenum */
+                if(sw->Index->swap_locdata)
+                {
+                    sortSwapLocData(epi);
+                }
+                if ( sw->verbose && totalwords > 10000 )  // just some random guess
+                {
+                    n++;
+                    percent = (n * 100)/totalwords;
+                    if (percent - lastPercent >= DELTA )
+                    {
+                        printf("\r  Writing word data: %3d%%", percent );
+                        fflush(stdout);
+                        lastPercent = percent;
+                    }
+                }
+                /* why check for stopwords here?  removestopwords could have remove them */
+                if ( !is_word_in_hash_table( indexf->header.hashstoplist, epi->word ) )                     /* Not a stopword */
+                {
+                    /* Build worddata buffer */
+                    build_worddata(sw, epi);
+                    /* let's see if word is already in the index */
+                    old_wordid = read_worddata(sw, epi, indexf, &buffer, &sz_buffer);
+                    /* If exists, we have to add the new worddata buffer to the old one */
+                    if(old_wordid)
+                    {
+                         add_worddata(sw, buffer, sz_buffer);
+                         efree(buffer);
+                         buffer = NULL;
+                         sz_buffer = 0;
+                         delete_worddata(sw, old_wordid, indexf);
+                         write_worddata(sw, epi, indexf);
+                         update_wordID(sw, epi, indexf);
+                    }
+                    else
+                    {
+                         /* Reset last error. It was set in read_worddata if
+                         ** word was not found */
+                         sw->lasterror = RC_OK;
+                         /* Write word to index file */
+                         write_worddata(sw, epi, indexf);
+                         write_word(sw, epi, indexf);
+                    }
+                }
+                epi = epi->next;
             }
         }
-    }    
-
+    }
     if (sw->verbose)
     {
         printf("\r  Writing word text: Complete\n" );
@@ -2920,7 +3105,6 @@ static void unSwapLocData(SWISH * sw, int idx_swap_file, ENTRY *ep)
     }
 }
 
-#ifndef USE_BTREE
 /* 2002-07 jmruiz - Sorts unswaped location data by metaname, filenum */
 static void sortSwapLocData(ENTRY *e)
 {
@@ -3001,4 +3185,3 @@ static void sortSwapLocData(ENTRY *e)
     efree(ptmp);
 
 }
-#endif
