@@ -1,17 +1,31 @@
 /*
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+$Id: merge.c,v 1.94 2005/05/12 15:41:05 karman Exp $
+
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 15:07:32 CDT 2005
+** added GPL
+    
 **-----------------------------------------------------------------
 **
 **  rewritten from scratch - moseley Oct 17, 2001
@@ -368,7 +382,10 @@ static void dup_header( SWISH *sw_input, SWISH *sw_output )
     read_header(sw_input, out_header, sw_input->indexlist->DB);
 
     out_header->totalfiles = 0;
+
+    /* $$$ This needs to be fixed */
     out_header->removedfiles = 0;
+    out_header->removed_word_positions = 0;
     out_header->totalwords = 0;
 
     freeMetaEntries( out_header );
@@ -579,7 +596,7 @@ static void make_meta_map( IndexFILE *in_index, SWISH *sw_output)
 *
 *****************************************************************************/
 
-static int  *sorted_data;
+static int  *sorted_data;  /* Static array to make the qsort function a bit quicker */
 
 static int     compnums(const void *s1, const void *s2)
 {
@@ -598,6 +615,13 @@ static int     compnums(const void *s1, const void *s2)
     return 0;
 }
 
+/******************************************************************************
+* load_filename_sort -
+*
+*   Creates an array used for sorting file names.
+*   Uses the pre-sorted array, if available, otherwise, creates one.
+*
+*******************************************************************************/
 
 static void load_filename_sort( SWISH *sw, IndexFILE *cur_index )
 {
@@ -621,7 +645,21 @@ static void load_filename_sort( SWISH *sw, IndexFILE *cur_index )
     cur_index->modified_meta = getPropNameByName( &cur_index->header, AUTOPROPERTY_LASTMODIFIED );
 
 
+    /*
+     * Since USE_PRESORT_ARRAY has a different internal format that what is generated
+     * by CreatePropeSortArray() we must ALWAYS create an actual integer
+     * array total_files long.
+     * 
+     * $$$ The problem is that with USE_PRESORT_ARRAY the format is different
+     *     before and after saving the array to disk
+     */
+
+#ifdef USE_PRESORT_ARRAY
+    if ( 1 )
+#else
     if ( !LoadSortedProps( cur_index, path_meta ) )
+#endif
+
     {
         FileRec fi;
         memset( &fi, 0, sizeof( FileRec ));
@@ -650,6 +688,7 @@ static void load_filename_sort( SWISH *sw, IndexFILE *cur_index )
 
     cur_index->path_order = sort_array;
 
+    /* $$$ can this be freeded when using BTREE??? */
     efree( path_meta->sorted_data );
     path_meta->sorted_data = NULL;
 }
