@@ -1,3 +1,31 @@
+/*
+$Id: metanames.h,v 1.24 2005/05/12 15:41:05 karman Exp $
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 18:19:34 CDT 2005
+** added GPL
+
+*/
+
 /* Jose Ruiz 2000/01 Definitions for MetaNames/Fields */
 
 /* META_INDEX and META_PROP could now share the same bit, since props and metas are separated entries */
@@ -9,7 +37,7 @@
 #define META_INTERNAL (1<<5)      /* flag saying this is an internal metaname */
 #define META_IGNORE_CASE (1<<6)   /* flag to say ignore case when comparing/sorting */
 #define META_NOSTRIP  (1<<7)      /* Do not strip low ascii chars when indexing */
-
+#define META_USE_STRCOLL (1<<8)  /* Use strcoll for sorting string properties */
 
 /* Macros to test the type of a MetaName */
 #define is_meta_internal(x)     ((x)->metaType & META_INTERNAL)
@@ -20,7 +48,7 @@
 #define is_meta_string(x)       ((x)->metaType & META_STRING)
 #define is_meta_ignore_case(x)  ((x)->metaType & META_IGNORE_CASE)
 #define is_meta_nostrip(x)      ((x)->metaType & META_NOSTRIP)
-
+#define is_meta_use_strcoll(x)  ((x)->metaType & META_USE_STRCOLL)
 
 int properties_compatible( struct metaEntry *m1, struct metaEntry *m2 );
 
