@@ -1,18 +1,30 @@
 /*
-$Id: parser.c,v 1.53 2004/11/22 21:08:56 whmoseley Exp $
-**
-**
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
-**
+$Id: parser.c,v 1.59 2005/05/12 15:41:05 karman Exp $
 **
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 14:54:24 CDT 2005
+** updated GPL
+****************************************************************************************
+
 ** 2001-09-21  new HTML parser using libxml2 (http://www.xmlsoft.org/) Bill Moseley
 **
 **  Parser reads from the stream in READ_CHUNK_SIZE (after an initial 4 byte chunk
@@ -875,7 +887,7 @@ static void Convert_to_latin1( PARSE_DATA *parse_data, char *txt, int txtlen )
 
         if ( ret == -2 )        // encoding failed
         {
-            if ( parse_data->sw->parser_warn_level >= 1 )
+            if ( parse_data->sw->parser_warn_level >= 3 )
                 xmlParserWarning(parse_data->ctxt, "Failed to convert internal UTF-8 to Latin-1.\nReplacing non ISO-8859-1 char with char '%c'\n", ENCODE_ERROR_CHAR);
 
 
@@ -883,12 +895,54 @@ static void Convert_to_latin1( PARSE_DATA *parse_data, char *txt, int txtlen )
 
 
             /* Skip one UTF-8 character -- returns null if not pointing to a UTF-8 char */
+
+            /*
+             * xmlUTF8Strpos() calls xmlUTFStrlen() which requires a null-terminated string.
+             * so, jump over the utf-8 char here.  Mostly from libxml2's encoding.c.
+             *
+             *
             if (  !(txt = (char *)xmlUTF8Strpos( (const xmlChar *)(&txt[inlen]), 1) ))
                 return;
+            */
+
+
+            {
+                char ch;
+                txt += inlen;  /* point to the start of the utf-8 char (where conversion left off) */
+
+                /* grab first utf-8 char and check that it's not null */
+                if ( 0 == (ch = *txt++) )
+                    return;
+
+                /* Make sure valid starting utf-8 char (must be 11xxxxxx) */
+                if ( 0xc0 != ( ch & 0xc0 ) )
+                    return;
+
+                /* Now skip over the aditional bytes based on the number of high-order bits */
+                while ( (ch <<= 1 ) & 0x80 )
+                {
+                    /* Are we past the end of the buffer? */
+                    if ( txt > end_buf )
+                    {
+                        if ( parse_data->sw->parser_warn_level >= 1 )
+                            xmlParserError(parse_data->ctxt, "Incomplete UTF-8 character found\n" );
+
+                        return;
+                    }
+
+                    /* all trailing bytes must be 10xx xxxx */
+                    if ( 0x80 != (*txt++ & 0xc0) )
+                    {
+                        if ( parse_data->sw->parser_warn_level >= 1 )
+                            xmlParserError(parse_data->ctxt, "Invalid UTF-8 sequence found. A secondary byte was: '0x%2x'\n", (unsigned char)txt[-1] );
+
+                        return;
+                    }
+                }
+            }
 
             /* Calculate the remaining length of the input string */
             inlen = (unsigned long)end_buf - (unsigned long)txt + 1;
-
             if ( inlen <= 0 )
                 return;
 
@@ -1715,7 +1769,6 @@ static void warning(void *data, const char *msg, ...)
     PARSE_DATA *parse_data = (PARSE_DATA *)data;
     char str[1000];
 
-    printf("8888\n");
     va_start(args, msg);
     vsnprintf(str, 1000, msg, args );
     va_end(args);
