@@ -1,19 +1,28 @@
 /*
-$Id: proplimit.c,v 1.42 2003/07/25 22:34:26 whmoseley Exp $
+$Id: proplimit.c,v 1.48 2005/05/12 15:41:05 karman Exp $
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 14:53:28 CDT 2005
+** added GPL
 **
 ** module to limit within a range of properties
 ** Created June 10, 2001 - moseley
@@ -34,6 +43,7 @@ $Id: proplimit.c,v 1.42 2003/07/25 22:34:26 whmoseley Exp $
 #include "result_sort.h"
 #include "swish_qsort.h"
 #include "proplimit.h"
+#include "array.h"
 
 
 // #define DEBUGLIMIT
@@ -582,7 +592,7 @@ static int create_lookup_array( IndexFILE *indexf, PROP_LIMITS *prop_limits, str
     for (i = 0; i < size; i++)
     {
         sort_array[i].filenum = i+1;
-        sort_array[i].sort = meta_entry->sorted_data[i];
+        sort_array[i].sort = DB_ReadSortedData( meta_entry->sorted_data, i );
     }
 
 
