@@ -1,25 +1,41 @@
 /*
-$Id: swish.c,v 1.126 2004/12/06 22:08:43 whmoseley Exp $
+$Id: swish.c,v 1.139 2006/06/19 01:27:16 augur Exp $
 **
 ** Swish Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
+
+** Mon May  9 11:07:38 CDT 2005
+** like swish2.c -- how much of this is really Kevin's original work?
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**---------------------------------------------------------
+
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+Mon May  9 10:57:22 CDT 2005 -- added GPL notice
+
+
 */
 
 
+
 #include <limits.h>     // for ULONG_MAX
 #include "swish.h"
 #include "swstring.h"
@@ -167,7 +183,7 @@ int     main(int argc, char **argv)
     SWISH  *sw;
     CMDPARAMS *params;
 
-    setlocale(LC_CTYPE, "");
+    setlocale(LC_ALL, "");
 
 
     /* Start a session */
@@ -256,7 +272,8 @@ static void    usage()
     printf("    swish [-e] [-i dir file ... ] [-S system] [-c file] [-f file] [-l] [-v (num)]\n");
     printf("    swish -w word1 word2 ... [-f file1 file2 ...] \\\n");
     printf("          [-P phrase_delimiter] [-p prop1 ...] [-s sortprop1 [asc|desc] ...] \\\n");
-    printf("          [-m num] [-t str] [-d delim] [-H (num)] [-x output_format] [-R rank_scheme]\n");
+    printf("          [-m num] [-t str] [-d delim] [-H (num)] [-x output_format] \\\n");
+    printf("          [-R rank_scheme] [-L prop low high]\n");
     printf("    swish -k (char|*) [-f file1 file2 ...]\n");
     printf("    swish -M index1 index2 ... outputfile\n");
     printf("    swish -N /path/to/compare/file\n");
@@ -287,6 +304,7 @@ static void    usage()
 
     printf("         -k : Print words starting with a given char.\n");
     printf("         -l : follow symbolic links when indexing\n");
+    printf("         -L : Limit results to a range of property values\n");
     printf("         -M : merges index files\n");
     printf("         -m : the maximum number of results to return [defaults to all results]\n");
     printf("         -N : index only files with a modification date newer than path supplied\n");
@@ -330,6 +348,7 @@ static void    usage()
     printf("         -V : prints the current version\n");
     printf("         -v : indexing verbosity level (0 to 3) [-v %d]\n", VERBOSE);
     printf("         -w : search for words \"word1 word2 ...\"\n");
+    printf("         -W : next param is ParserWarnLevel [-W 2]\n");
     printf("         -x : \"Extended Output Format\": Specify the output format.\n");
     printf("\n");
     printf("version: %s\n docs: http://swish-e.org\n Scripts and Modules at: (libexecdir) = %s\n", VERSION, get_libexec());
@@ -352,50 +371,27 @@ static void    printversion()
 /*************************************************************************
 * swish_new -- create a general purpose swish structure
 *
-*  NOTE: ANY CHANGES HERE SHOULD ALSO BE MADE IN swish2.c:SwishNew()
-*  This is called when using the binrary.
 *  Note that initModule_* code is called even when it's not going to be used
 *  (e.g. initModule_HTTP is called when searching).
 *
-*  SwishNew is search related only
 *
 **************************************************************************/
 
 static SWISH  *swish_new()
 
 {
-    SWISH  *sw;
+    SWISH  *sw = SwishNew();
 
-    sw = emalloc(sizeof(SWISH));
-    memset(sw, 0, sizeof(SWISH));
 
-    initModule_DB(sw);
+    /* Additional modules needed for indexin (which we are not sure about yet... */
     initModule_ResultSort(sw);
-    initModule_Swish_Words(sw);  /* allocate a buffer */
-
     initModule_Filter(sw);
-    initModule_Entities(sw);
+    initModule_Entities(sw);  /* used only by the old HTML parser -- not long to live */
     initModule_Index(sw);
     initModule_FS(sw);
     initModule_HTTP(sw);
     initModule_Prog(sw);
 
-
-    sw->lasterror = RC_OK;
-    sw->lasterrorstr[0] = '\0';
-    sw->verbose = VERBOSE;
-    sw->headerOutVerbose = 1;
-    sw->DefaultDocType = NODOCTYPE;
-
-#ifdef HAVE_ZLIB
-    sw->PropCompressionLevel = Z_DEFAULT_COMPRESSION;
-#endif
-
-
-
-
-    /* Make rest of lookup tables */
-    makeallstringlookuptables(sw);
     return (sw);
 }
 
@@ -628,6 +624,9 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
 {
     char c;
     char *w;
+#if defined(_WIN32) || defined(__CYGWIN__)
+    volatile unsigned int DEBUG_MASK_HACK;
+#endif
 
 
 
@@ -682,6 +681,7 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
             case 'S':  /* data Source */
             case 'c':  /* config file */
             case 'v':  /* verbose indexing - not really limited to indexing */
+            case 'W':  /* ParserWarnLevel - also configurable in conf file */
             case 'N':  /* limit by date */
             case 'l':  /* follow symbolic links */
             case 'e':  /* economy indexing mode (also for merge) */
@@ -773,7 +773,17 @@ static void get_command_line_params(SWISH *sw, char **argv, CMDPARAMS *params )
 
                 }
 
+#if defined(_WIN32) || defined(__CYGWIN__)
+    /* 
+     * DEBUG_MASK: can't sum constants imported from a DLL
+     * (see: "man ld" under --enable-auto-import)
+     * 2005-05-12 - David L Norris 
+     */
+     DEBUG_MASK_HACK = DEBUG_MASK;
+		if ( DEBUG_MASK_HACK & DEBUG_LIST_FUZZY )
+#else
                 if ( DEBUG_MASK & DEBUG_LIST_FUZZY )
+#endif
                 {
                     dump_fuzzy_list();
                     exit(1);
@@ -956,6 +966,12 @@ static char **fetch_indexing_params(SWISH *sw, char **argv, CMDPARAMS *params, c
             sw->verbose = get_param_number( &argv, switch_char );
             break;
         }
+        
+        
+        /* ParserWarnLevel */
+        case 'W':
+            sw->parser_warn_level = get_param_number( &argv, switch_char );
+            break;
 
 
 
@@ -1234,6 +1250,7 @@ static char **fetch_search_params(SWISH *sw, char **argv, CMDPARAMS *params, cha
         case 'R':
             sw->RankScheme = get_param_number( &argv, switch_char );
             break;
+            
 
         /* Ignore sorted indexes */
 
@@ -1338,30 +1355,20 @@ static void cmd_index( SWISH *sw, CMDPARAMS *params )
 
     /* Check for UPDATE_MODE jmruiz 2002/03 */
     if ( MODE_UPDATE == params->run_mode || MODE_REMOVE == params->run_mode )
+#ifndef USE_BTREE
+        progerr("Invalid operation mode '%d': Update mode only supported with USE_BTREE feature", (int)params->run_mode);
+#else
     {
         /* Set update_mode */
         sw->Index->update_mode = params->run_mode;
 
-        /* Open the index file for read/write */
-        sw->indexlist->DB = (void *) DB_Open(sw, sw->indexlist->line,DB_READWRITE);
-        if ( sw->lasterror )
+        if ( !open_single_index( sw, sw->indexlist, DB_READWRITE ) )
             SwishAbortLastError( sw );
 
-
-        /* Read the header and overwrite the '-c' option and default values -
-        ** In other words, the header values are the good ones */
-        read_header(sw, &sw->indexlist->header, sw->indexlist->DB);
-        sw->TotalWords = sw->indexlist->header.totalwords;
-        sw->TotalFiles = sw->indexlist->header.totalfiles;
-        sw->TotalWordPos = sw->indexlist->header.total_word_positions;
-
-        /* Adjust filenum to totalfiles */
-        sw->Index->filenum = sw->TotalFiles;
-
-#ifndef USE_BTREE
-        progerr("Invalid operation mode '%d': Update mode only supported with USE_BTREE feature", (int)params->run_mode);
-#endif
+        /* Adjust file number to start after the last file number in the index */
+        sw->Index->filenum = sw->indexlist->header.totalfiles;
     }
+#endif
 
 
     else
@@ -1594,7 +1601,7 @@ static void cmd_search( SWISH *sw, CMDPARAMS *params )
 
 static void write_index_file( SWISH *sw, int process_stopwords, double elapsedStart, double cpuStart, int merge)
 {
-    int totalfiles = getfilecount(sw->indexlist);
+    int totalfiles = getfilecount(sw->indexlist) - sw->indexlist->header.removedfiles;  /* just for display */
     int stopwords = 0;
     struct swline *cur_line;
 
@@ -1665,7 +1672,7 @@ static void write_index_file( SWISH *sw, int process_stopwords, double elapsedSt
 
     fflush(stdout);
 
-    write_header(sw, &sw->indexlist->header, sw->indexlist->DB, sw->indexlist->line, sw->indexlist->header.totalwords, totalfiles, sw->indexlist->total_word_positions, sw->indexlist->header.removedfiles, merge);
+    write_header( sw, merge );
 
     fflush(stdout);
 
@@ -1677,7 +1684,10 @@ static void write_index_file( SWISH *sw, int process_stopwords, double elapsedSt
 
 
     if (sw->verbose)
-        printf("%s unique word%s indexed.\n", comma_long( sw->indexlist->header.totalwords ), (sw->indexlist->header.totalwords == 1) ? "" : "s");
+    {
+        int totalwords = sw->indexlist->header.totalwords;
+        printf("%s unique word%s indexed.\n", comma_long( totalwords ), (totalwords == 1) ? "" : "s");
+    }
 
 
     /* Sort properties -> Better search performance */
@@ -1701,12 +1711,13 @@ static void write_index_file( SWISH *sw, int process_stopwords, double elapsedSt
         if (totalfiles)
         {
             printf("%s file%s indexed.  ",
-                    comma_long( totalfiles ), (totalfiles == 1) ? "" : "s");
+            comma_long( totalfiles ), (totalfiles == 1) ? "" : "s"); /* common_long is not thread safe -- shares memory */
 
             printf("%s total bytes.  ", comma_long(sw->indexlist->total_bytes) );
 
-            printf("%s total words.\n", comma_long(sw->indexlist->total_word_positions) );
+            printf("%s total words.\n", comma_long(sw->indexlist->total_word_positions_cur_run) );
         }
+
         else
             printf("no files indexed.\n");
 
