@@ -1,23 +1,33 @@
 /*
-** $Id: swish.h,v 1.202 2004/12/06 22:08:43 whmoseley Exp $
+** $Id: swish.h,v 1.209 2007/01/05 22:14:10 whmoseley Exp $
 **
 ** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**-------------------------------------------------
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+** Mon May  9 18:18:48 CDT 2005
+** added GPL
+***************************************************************************************
 ** Added support for METADATA
 ** G. Hill  ghill@library.berkeley.edu   3/18/97
 **
@@ -147,9 +157,9 @@ binary and the index.
 checked in db_native.c (DB_CheckHeader routine) */
 
 #ifdef USE_BTREE
-#define SWISH_MAGIC 29031972L
+#define SWISH_MAGIC 05052004L
 #else
-#define SWISH_MAGIC 39031972L
+#define SWISH_MAGIC 11282006L
 #endif
 
 #define INDEXFILE "index.swish-e"
@@ -264,6 +274,8 @@ checked in db_native.c (DB_CheckHeader routine) */
 #define TOTALWORDSPERFILE_ID (BASEHEADER + 30)  /* total words per file array */
 #endif
 
+#define TOTALWORDS_REMOVED_ID (BASEHEADER + 31) /* 2005-01-14 for tracking total words removed */
+
 /* -- end of headers */
 
 #define MAXFILELEN 1000
@@ -407,6 +419,7 @@ typedef struct
 typedef struct
 {
     FILE   *fp;                 /* may be also a filter stream or NULL if not opened */
+    pid_t  filter_pid;          /* process id of filter program, if forked */
     char   *real_path;          /* path/URL to indexed file - may be modified by ReplaceRules */
     char   *orig_path;          /* original path provided to swish */
     char   *work_path;          /* path to file to index (may be tmpfile or real_path) */
@@ -545,7 +558,7 @@ typedef struct
     FUZZY_OBJECT *fuzzy_data;
 
     /* Total files and words in index file */
-    int     totalwords;
+    int     totalwords;    /* Total *unique* words */
     int     totalfiles;
     int     removedfiles;
 
@@ -590,7 +603,7 @@ typedef struct
     int     metaCounter;        /* Number of metanames */
 
     int     total_word_positions;       /* IDF ranking */
-
+    int     removed_word_positions;     /* total words (not just unique words) */
 }
 INDEXDATAHEADER;
 
@@ -604,7 +617,7 @@ typedef struct IndexFILE
     char   *line;               /* Name of the index file */
 
     unsigned long total_bytes;  /* Just to show total size when indexing */
-    unsigned long total_word_positions;
+    unsigned long total_word_positions_cur_run;  /* count *while* indexing */
 
 
 
@@ -834,7 +847,7 @@ typedef struct SWISH
 
 
     /* Total words and files in all index files */
-    int     TotalWords;
+    /* int     TotalWords;  Total *unique words*  $$$ doesn't seem to be used */
     int     TotalFiles;
 
     /* verbose flag */
@@ -1008,6 +1021,7 @@ extern struct _indexing_data_source_def *IndexingDataSource;
 void    allocatedefaults(void);
 
 int SwishAttach(SWISH *);
+int open_single_index( SWISH *sw, IndexFILE *indexf, int db_mode );
 SWISH  *SwishNew(void);
 void    SwishFree(SWISH *);
 
