@@ -1,23 +1,40 @@
 /*
-** $Id: swish2.c,v 1.101 2004/08/31 03:15:06 karman Exp $
+** $Id: swish2.c,v 1.108 2005/05/12 15:41:11 karman Exp $
 **
 ** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
 **
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
+
+** Mon May  9 11:06:18 CDT 2005
+karman : NOTE that none of this looks like Kevin's code. Is it all Bill's
+and simply mis-labeled?
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+Mon May  9 10:57:22 CDT 2005 -- added GPL notice
+
+
 */
 
+
 #include "swish.h"
 #include "swstring.h"
 #include "mem.h"
@@ -48,9 +65,6 @@ unsigned int DEBUG_MASK = 0;
 /*************************************************************************
 * SwishNew -- create a search swish structure
 *
-*  NOTE: ANY CHANGES HERE SHOULD ALSO BE MADE IN swish.c swish_new
-*
-*  $$$ Please join these routines!
 *
 **************************************************************************/
 
@@ -74,8 +88,9 @@ SWISH  *SwishNew()
     sw->lasterror = RC_OK;
     sw->lasterrorstr[0] = '\0';
     sw->verbose = VERBOSE;
+    /* karman added -W opt which will override warn_level default at cmd line */
+    sw->parser_warn_level = 2; /* report if libxml2 aborts processing a document. */
     sw->headerOutVerbose = 1;
-    
     sw->DefaultDocType = NODOCTYPE;
 
 #ifdef HAVE_ZLIB
@@ -199,7 +214,7 @@ void    SwishClose(SWISH * sw)
 
 
 /*************************************************************************
-* SwishInit -- create a swish hanlde
+* SwishInit -- create a swish handle for the indexe or indexes passed in
 *
 *
 **************************************************************************/
@@ -217,7 +232,7 @@ SWISH  *SwishInit(char *indexfiles)
         set_progerr(INDEX_FILE_ERROR, sw, "No index file supplied" );
         return sw;
     }
-    
+
 
     /* Parse out index files, and append to indexlist */
     sl = parse_line(indexfiles);
@@ -228,7 +243,7 @@ SWISH  *SwishInit(char *indexfiles)
         return sw;
     }
 
-    
+
 
     for (i = 0; i < sl->n; i++)
         addindexfile(sw, sl->word[i]);
@@ -249,42 +264,65 @@ SWISH  *SwishInit(char *indexfiles)
 * SwishAttach - Connect to the database
 *  This just opens the index files
 *
+*  Maybe this could be passed a variable length of arguments
+*  so that swis.c:cmd_index() could call SwishAttach( sw, DB_READWRITE )
+*  to have all similar code in one place.
+*
 * Returns false on Failure
 **************************************************/
 
 int     SwishAttach(SWISH * sw)
 {
-    IndexFILE *indexlist;
-
+    IndexFILE *indexlist = sw->indexlist;  /* head of list of indexes */
     IndexFILE *tmplist;
- 
-    indexlist = sw->indexlist;
-    sw->TotalWords = 0;
-    sw->TotalFiles = 0;
 
 
-    /* First of all . Read header default values from all index fileis */
+    /* First of all . Read header default values from all index files */
     /* With this, we read wordchars, stripchars, ... */
     for (tmplist = indexlist; tmplist;)
-    {
-
-        tmplist->DB = (void *)DB_Open(sw, tmplist->line, DB_READ);
-        if ( sw->lasterror )
+        if ( !open_single_index( sw, tmplist, DB_READ ) )
             return 0;
-
-        read_header(sw, &tmplist->header, tmplist->DB);
+        else
+            tmplist = tmplist->next;
 
 
-        sw->TotalWords += tmplist->header.totalwords;
-        sw->TotalFiles += tmplist->header.totalfiles;
-	sw->TotalWordPos += tmplist->header.total_word_positions;
-	
-        tmplist = tmplist->next;
-    }
-
-    return ( sw->lasterror == 0 ); 
+    return ( sw->lasterror == 0 );
 }
 
+/****************************************************************************
+* open_single_index -- opens the index and reads in its header data
+*
+* Pass:
+*   sw
+*   indexf
+*   db_mode  - open in read or read/write
+*
+* Returns
+*   true if ok.
+*
+****************************************************************************/
+
+int open_single_index( SWISH *sw, IndexFILE *indexf, int db_mode )
+{
+    INDEXDATAHEADER *header = &indexf->header;
+
+
+    indexf->DB = (void *)DB_Open(sw, indexf->line, db_mode);
+
+    if ( sw->lasterror )
+        return 0;
+
+    read_header(sw, header, indexf->DB);
+
+    /* These values are used in ranking */
+
+    sw->TotalFiles   += header->totalfiles - header->removedfiles;
+    sw->TotalWordPos += header->total_word_positions - header->removed_word_positions;
+
+    return 1;
+}
+
+
 /********************************************************************************
 * SwishSetRefPtr - for use the SWISH::API to save the SV* of the swish handle
 *
