@@ -1,20 +1,30 @@
 /*
-$Id: swish_words.c,v 1.37.2.2 2003/12/14 01:26:55 whmoseley Exp $
-**
-** This program and library is free software; you can redistribute it and/or
-** modify it under the terms of the GNU (Library) General Public License
-** as published by the Free Software Foundation; either version 2
-** of the License, or any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU (Library) General Public License for more details.
-**
-** You should have received a copy of the GNU (Library) General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-**
+$Id: swish_words.c,v 1.46 2006/08/27 20:22:54 karman Exp $
+
+
+    This file is part of Swish-e.
+
+    Swish-e is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    Swish-e is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along  with Swish-e; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    
+    See the COPYING file that accompanies the Swish-e distribution for details
+    of the GNU GPL and the special exception available for linking against
+    the Swish-e library.
+    
+Mon May  9 10:57:22 CDT 2005 -- added GPL notice
+
+
 **
 ** 2001-05-23  moseley  created - replaced parser in search.c
 **
@@ -112,8 +122,8 @@ void freeModule_Swish_Words (SWISH *sw)
 static int isSearchOperatorChar( int c, int phrase_delimiter, int inphrase )
 {
     return inphrase
-        ? ( '*' == c || c == phrase_delimiter )
-        : ( '(' == c || ')' == c || '=' == c || '*' == c || c == phrase_delimiter );
+        ? ( '*' == c || '?' == c || c == phrase_delimiter )
+        : ( '(' == c || ')' == c || '=' == c || '*' == c || '?' == c || c == phrase_delimiter );
 }
 
 
@@ -182,6 +192,23 @@ static int next_token( char **buf, char **word, int *lenword, int phrase_delimit
             (*word)[i++] = **buf;  /* can't this be done in one line? */
             (*buf)++;
         }
+        
+        else if ( **buf == '?' )
+        {
+            /* ? is a search operator so fails first test above
+               but we want to pass it through like a normal word char
+               if it is not the first character in the word
+             */
+             
+            if(!i)
+                return WILDCARD_NOT_ALLOWED_AT_WORD_START;
+                
+            backslash = 0;
+            
+            (*word)[i++] = **buf;  /* can't this be done in one line? */
+            (*buf)++;
+                
+        }
 
         else  /* this is a search operator char */
         {
@@ -200,7 +227,6 @@ static int next_token( char **buf, char **word, int *lenword, int phrase_delimit
 
             }
 
-
             (*word)[i++] = **buf;  /* save the search operator char as it's own token, and end. */
             (*buf)++;
             break;
@@ -224,6 +250,13 @@ static int next_token( char **buf, char **word, int *lenword, int phrase_delimit
 static int next_swish_word(INDEXDATAHEADER *header, char **buf, char **word, int *lenword )
 {
     int     i;
+    
+    /* Also set flag for "?" (wildcard), in general set and at end of a term/word
+     * At start there is never a wildcard allowed, because of sequential lookup in index
+     * performance issue !!
+     */
+    header->wordcharslookuptable[63] = 1;
+    header->endcharslookuptable[63] = 1;
 
     /* skip non-wordchars */
     while ( **buf && !header->wordcharslookuptable[tolower((unsigned char)(**buf))] )
@@ -314,7 +347,7 @@ static struct swline *parse_swish_words( SWISH *sw, INDEXDATAHEADER *header, cha
                 /* yuck! */
                 swish_words = (struct swline *) addswline( swish_words, "(" );
                 swish_words = (struct swline *) addswline( swish_words, fw->string_list[0] );
-                swish_words = (struct swline *) addswline( swish_words, "or" );
+                swish_words = (struct swline *) addswline( swish_words, OR_WORD );
                 swish_words = (struct swline *) addswline( swish_words, fw->string_list[1] );
                 swish_words = (struct swline *) addswline( swish_words, ")" );
             }
@@ -435,6 +468,9 @@ static char *isBooleanOperatorWord( char * word )
     /* don't need strcasecmp here, since word should alrady be lowercase -- need to check alt-search first */
     if (!strcasecmp( word, _AND_WORD))
         return AND_WORD;
+        
+    if (!strncasecmp( word, _NEAR_WORD, strlen(_NEAR_WORD)))
+        return NEAR_WORD;
 
     if (!strcasecmp( word, _OR_WORD))
         return OR_WORD;
@@ -454,6 +490,9 @@ static char *isBooleanOperator( char * word )
 {
     if (!strcasecmp( word, AND_WORD))
         return _AND_WORD;
+        
+    if (!strncasecmp( word, NEAR_WORD, strlen(NEAR_WORD)))
+        return NEAR_WORD;
 
     if (!strcasecmp( word, OR_WORD))
         return _OR_WORD;
@@ -483,9 +522,6 @@ static void switch_back_operators( struct swline *sl )
 
 
 
-
-
-
 static struct swline *tokenize_query_string( SEARCH_OBJECT *srch, char *words, INDEXDATAHEADER *header )
 {
     char   *curpos;               /* current position in the words string */
@@ -582,6 +618,7 @@ static struct swline *tokenize_query_string( SEARCH_OBJECT *srch, char *words, I
         if ( !inphrase )
         {
             char *operator, *nextoperator;
+            char nearop[100];
 
             if ( (operator = isBooleanOperatorWord( temp->line )) )
             {
@@ -602,9 +639,13 @@ static struct swline *tokenize_query_string( SEARCH_OBJECT *srch, char *words, I
                     /* Remove the "and" word */
                     replace_swline( &tokens, andword, (struct swline *)NULL ); /* cut it out */
                 }
+                
+                strcpy(nearop, operator);
+                if (!strncasecmp( operator, NEAR_WORD, strlen(NEAR_WORD)))
+                    strcat(nearop, temp->line + strlen(_NEAR_WORD));
 
                 /* Replace the string with the operator string */
-                new = newswline(operator);
+                new = newswline(nearop);
                 new->other.nodep = new;  // Group of 1 node (last is itself)
 
                 replace_swline( &tokens, temp, new ); /* change it */
@@ -741,7 +782,7 @@ struct swline *parse_swish_query( DB_RESULTS *db_results )
 
 static int     isrule(char *word)
 {
-    if (!strcmp(word, AND_WORD) || !strcmp(word, OR_WORD) || !strcmp(word, NOT_WORD))
+    if (!strcmp(word, AND_WORD) || !strncmp(word, NEAR_WORD, strlen(NEAR_WORD)) || !strcmp(word, OR_WORD) || !strcmp(word, NOT_WORD))
         return 1;
     else
         return 0;
